# Copyright (c) 2023 Maxwell G <maxwell@gtmx.me>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Helpers to respawn a module to run using the system interpreter
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

try:
    from ansible.module_utils.common import respawn
except ImportError:
    HAS_RESPAWN_UTIL = False
else:
    HAS_RESPAWN_UTIL = True


SYSTEM_PYTHON_INTERPRETERS = (
    "/usr/bin/libexec/platform-python",
    "/usr/bin/python3",
    "/usr/bin/python2",
    "/usr/bin/python",
)


def respawn_module(module):
    """
    Respawn an ansible module to using the first interpreter in
    SYSTEM_PYTHON_INTERPRETERS that contains `module`.

    Args:
        module (str): Name of python module to search for

    Returns:
        Returns None if the module cannot be respawned.
    """
    if respawn.has_respawned():
        return
    interpreter = respawn.probe_interpreters_for_module(
        SYSTEM_PYTHON_INTERPRETERS, module
    )
    if interpreter:
        respawn.respawn_module(interpreter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_set_cloud_services
short_description: Set the connection settings between the Management Server and Check Point's Infinity Portal.
description:
  - Set the connection settings between the Management Server and Check Point's Infinity Portal.
  - All operations are performed over Web Services API.
  - Available from R81.20 management version.
version_added: "5.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  gateways_onboarding_settings:
    description:
      - Gateways on-boarding to Infinity Portal settings.
    type: dict
    suboptions:
      connection_method:
        description:
          - Indicate whether Gateways will be connected to Infinity Portal automatically or only after policy installation.
        type: str
        choices: ['automatically', 'after install policy']
      participant_gateways:
        description:
          - Which Gateways will be connected to Infinity Portal.
        type: str
        choices: ['all', 'specific']
      specific_gateways:
        description:
          - Selection of targets identified by the name or UID which will be on-boarded to the cloud. Configuration will be applied only when
            "participant-gateways" field is set to "specific".
        type: list
        elements: str
      details_level:
        description:
          - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
            representation of the object.
        type: str
        choices: ['uid', 'standard', 'full']
  status:
    description:
      - Connection status.
    type: str
    choices: ['connected', 'disabled']
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: set-cloud-services
  cp_mgmt_set_cloud_services:
    gateways_onboarding_settings:
      connection_method: after install policy
      participant_gateways: specific
      specific_gateways: gw1
"""

RETURN = """
cp_mgmt_set_cloud_services:
  description: The checkpoint set-cloud-services output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        gateways_onboarding_settings=dict(type='dict', options=dict(
            connection_method=dict(type='str', choices=['automatically', 'after install policy']),
            participant_gateways=dict(type='str', choices=['all', 'specific']),
            specific_gateways=dict(type='list', elements='str'),
            details_level=dict(type='str', choices=['uid', 'standard', 'full'])
        )),
        status=dict(type='str', choices=['connected', 'disabled'])
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "set-cloud-services"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#
# (c) 2018 Red Hat, Inc.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type
import json

from contextlib import contextmanager
from copy import deepcopy


try:
    from lxml.etree import fromstring, tostring
except ImportError:
    from xml.etree.ElementTree import fromstring, tostring

from ansible.module_utils.common.text.converters import to_bytes, to_text
from ansible.module_utils.connection import Connection, ConnectionError

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.netconf import (
    NetconfConnection,
)


IGNORE_XML_ATTRIBUTE = ()


def get_connection(module):
    if hasattr(module, "_netconf_connection"):
        return module._netconf_connection

    capabilities = get_capabilities(module)
    network_api = capabilities.get("network_api")
    if network_api == "netconf":
        module._netconf_connection = NetconfConnection(module._socket_path)
    else:
        module.fail_json(msg="Invalid connection type %s" % network_api)

    return module._netconf_connection


def get_capabilities(module):
    if hasattr(module, "_netconf_capabilities"):
        return module._netconf_capabilities

    capabilities = Connection(module._socket_path).get_capabilities()
    module._netconf_capabilities = json.loads(capabilities)
    return module._netconf_capabilities


def lock_configuration(module, target=None):
    conn = get_connection(module)
    return conn.lock(target=target)


def unlock_configuration(module, target=None):
    conn = get_connection(module)
    return conn.unlock(target=target)


@contextmanager
def locked_config(module, target=None):
    try:
        lock_configuration(module, target=target)
        yield
    finally:
        unlock_configuration(module, target=target)


def get_config(module, source, filter=None, lock=False):
    conn = get_connection(module)
    locked = False

    try:
        if lock:
            conn.lock(target=source)
            locked = True
        response = conn.get_config(source=source, filter=filter)

    except ConnectionError as e:
        module.fail_json(msg=to_text(e, errors="surrogate_then_replace").strip())

    finally:
        if locked:
            conn.unlock(target=source)

    return response


def get(module, filter, lock=False):
    conn = get_connection(module)
    locked = False

    try:
        if lock:
            conn.lock(target="running")
            locked = True

        response = conn.get(filter=filter)

    except ConnectionError as e:
        module.fail_json(msg=to_text(e, errors="surrogate_then_replace").strip())

    finally:
        if locked:
            conn.unlock(target="running")

    return response


def dispatch(module, request):
    conn = get_connection(module)
    try:
        response = conn.dispatch(request)
    except ConnectionError as e:
        module.fail_json(msg=to_text(e, errors="surrogate_then_replace").strip())

    return response


def sanitize_xml(data):
    tree = fromstring(to_bytes(deepcopy(data), errors="surrogate_then_replace"))
    for element in tree.iter():
        # remove attributes
        attribute = element.attrib
        if attribute:
            for key in list(attribute):
                if key not in IGNORE_XML_ATTRIBUTE:
                    attribute.pop(key)
    return to_text(tostring(tree), errors="surrogate_then_replace").strip()



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on containerd's platforms Go module
# (https://github.com/containerd/containerd/tree/main/platforms)
#
# Copyright (c) 2023 Felix Fontein <felix@fontein.de>
# Copyright The containerd Authors
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import re
import typing as t

_VALID_STR = re.compile("^[A-Za-z0-9_-]+$")


def _validate_part(string: str, part: str, part_name: str) -> str:
    if not part:
        raise ValueError(f'Invalid platform string "{string}": {part_name} is empty')
    if not _VALID_STR.match(part):
        raise ValueError(
            f'Invalid platform string "{string}": {part_name} has invalid characters'
        )
    return part


# See https://github.com/containerd/containerd/blob/main/platforms/database.go#L32-L38
_KNOWN_OS = (
    "aix",
    "android",
    "darwin",
    "dragonfly",
    "freebsd",
    "hurd",
    "illumos",
    "ios",
    "js",
    "linux",
    "nacl",
    "netbsd",
    "openbsd",
    "plan9",
    "solaris",
    "windows",
    "zos",
)

# See https://github.com/containerd/containerd/blob/main/platforms/database.go#L54-L60
_KNOWN_ARCH = (
    "386",
    "amd64",
    "amd64p32",
    "arm",
    "armbe",
    "arm64",
    "arm64be",
    "ppc64",
    "ppc64le",
    "loong64",
    "mips",
    "mipsle",
    "mips64",
    "mips64le",
    "mips64p32",
    "mips64p32le",
    "ppc",
    "riscv",
    "riscv64",
    "s390",
    "s390x",
    "sparc",
    "sparc64",
    "wasm",
)


def _normalize_os(os_str: str) -> str:
    # See normalizeOS() in https://github.com/containerd/containerd/blob/main/platforms/database.go
    os_str = os_str.lower()
    if os_str == "macos":
        os_str = "darwin"
    return os_str


_NORMALIZE_ARCH = {
    ("i386", None): ("386", ""),
    ("x86_64", "v1"): ("amd64", ""),
    ("x86-64", "v1"): ("amd64", ""),
    ("amd64", "v1"): ("amd64", ""),
    ("x86_64", None): ("amd64", None),
    ("x86-64", None): ("amd64", None),
    ("amd64", None): ("amd64", None),
    ("aarch64", "8"): ("arm64", ""),
    ("arm64", "8"): ("arm64", ""),
    ("aarch64", "v8"): ("arm64", ""),
    ("arm64", "v8"): ("arm64", ""),
    ("aarch64", None): ("arm64", None),
    ("arm64", None): ("arm64", None),
    ("armhf", None): ("arm", "v7"),
    ("armel", None): ("arm", "v6"),
    ("arm", ""): ("arm", "v7"),
    ("arm", "5"): ("arm", "v5"),
    ("arm", "6"): ("arm", "v6"),
    ("arm", "7"): ("arm", "v7"),
    ("arm", "8"): ("arm", "v8"),
    ("arm", None): ("arm", None),
}


def _normalize_arch(arch_str: str, variant_str: str) -> tuple[str, str]:
    # See normalizeArch() in https://github.com/containerd/containerd/blob/main/platforms/database.go
    arch_str = arch_str.lower()
    variant_str = variant_str.lower()
    res = _NORMALIZE_ARCH.get((arch_str, variant_str))
    if res is None:
        res = _NORMALIZE_ARCH.get((arch_str, None))
    if res is None:
        return arch_str, variant_str
    arch_str = res[0]
    if res[1] is not None:
        variant_str = res[1]
    return arch_str, variant_str


class _Platform:
    def __init__(
        self, os: str | None = None, arch: str | None = None, variant: str | None = None
    ) -> None:
        self.os = os
        self.arch = arch
        self.variant = variant
        if variant is not None:
            if arch is None:
                raise ValueError("If variant is given, architecture must be given too")
            if os is None:
                raise ValueError("If variant is given, os must be given too")

    @classmethod
    def parse_platform_string(
        cls,
        string: str | None,
        daemon_os: str | None = None,
        daemon_arch: str | None = None,
    ) -> t.Self:
        # See Parse() in https://github.com/containerd/containerd/blob/main/platforms/platforms.go
        if string is None:
            return cls()
        if not string:
            raise ValueError("Platform string must be non-empty")
        parts = string.split("/", 2)
        arch = None
        variant = None
        if len(parts) == 1:
            _validate_part(string, string, "OS/architecture")
            # The part is either OS or architecture
            os = _normalize_os(string)
            if os in _KNOWN_OS:
                if daemon_arch is not None:
                    arch, variant = _normalize_arch(daemon_arch, "")
                return cls(os=os, arch=arch, variant=variant)
            arch, variant = _normalize_arch(os, "")
            if arch in _KNOWN_ARCH:
                return cls(
                    os=_normalize_os(daemon_os) if daemon_os else None,
                    arch=arch or None,
                    variant=variant or None,
                )
            raise ValueError(
                f'Invalid platform string "{string}": unknown OS or architecture'
            )
        os = _validate_part(string, parts[0], "OS")
        if not os:
            raise ValueError(f'Invalid platform string "{string}": OS is empty')
        arch = (
            _validate_part(string, parts[1], "architecture") if len(parts) > 1 else None
        )
        if arch is not None and not arch:
            raise ValueError(
                f'Invalid platform string "{string}": architecture is empty'
            )
        variant = (
            _validate_part(string, parts[2], "variant") if len(parts) > 2 else None
        )
        if variant is not None and not variant:
            raise ValueError(f'Invalid platform string "{string}": variant is empty')
        assert arch is not None  # otherwise variant would be None as well
        arch, variant = _normalize_arch(arch, variant or "")
        if len(parts) == 2 and arch == "arm" and variant == "v7":
            variant = None
        if len(parts) == 3 and arch == "arm64" and variant == "":
            variant = "v8"
        return cls(os=_normalize_os(os), arch=arch, variant=variant or None)

    def __str__(self) -> str:
        if self.variant:
            assert (
                self.os is not None and self.arch is not None
            )  # ensured in constructor
            parts: list[str] = [self.os, self.arch, self.variant]
        elif self.os:
            if self.arch:
                parts = [self.os, self.arch]
            else:
                parts = [self.os]
        elif self.arch is not None:
            parts = [self.arch]
        else:
            parts = []
        return "/".join(parts)

    def __repr__(self) -> str:
        return (
            f"_Platform(os={self.os!r}, arch={self.arch!r}, variant={self.variant!r})"
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, _Platform):
            return NotImplemented
        return (
            self.os == other.os
            and self.arch == other.arch
            and self.variant == other.variant
        )


def normalize_platform_string(
    string: str, daemon_os: str | None = None, daemon_arch: str | None = None
) -> str:
    return str(
        _Platform.parse_platform_string(
            string, daemon_os=daemon_os, daemon_arch=daemon_arch
        )
    )


def compose_platform_string(
    os: str | None = None,
    arch: str | None = None,
    variant: str | None = None,
    daemon_os: str | None = None,
    daemon_arch: str | None = None,
) -> str:
    if os is None and daemon_os is not None:
        os = _normalize_os(daemon_os)
    if arch is None and daemon_arch is not None:
        arch, variant = _normalize_arch(daemon_arch, variant or "")
        variant = variant or None
    return str(_Platform(os=os, arch=arch, variant=variant or None))


def compare_platform_strings(string1: str, string2: str) -> bool:
    return _Platform.parse_platform_string(string1) == _Platform.parse_platform_string(
        string2
    )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: directconnect_confirm_connection
short_description: Confirms the creation of a hosted DirectConnect connection
description:
  - Confirms the creation of a hosted DirectConnect, which requires approval before it can be used.
  - DirectConnect connections that require approval would be in the C(ordering).
  - After confirmation, they will move to the C(pending) state and finally the C(available) state.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_direct_connect_confirm_connection).
    The usage did not change.
author:
  - "Matt Traynham (@mtraynham)"
options:
  name:
    description:
      - The name of the Direct Connect connection.
      - One of I(connection_id) or I(name) must be specified.
    type: str
  connection_id:
    description:
      - The ID of the Direct Connect connection.
      - One of I(connection_id) or I(name) must be specified.
    type: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""

# confirm a Direct Connect by name
- name: confirm the connection id
  community.aws.directconnect_confirm_connection:
    name: my_host_direct_connect

# confirm a Direct Connect by connection_id
- name: confirm the connection id
  community.aws.directconnect_confirm_connection:
    connection_id: dxcon-xxxxxxxx
"""

RETURN = r"""

connection_state:
  description: The state of the connection.
  returned: always
  type: str
  sample: pending
"""

import traceback

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # handled by imported AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.direct_connect import DirectConnectError
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule

retry_params = {"retries": 10, "delay": 5, "backoff": 1.2, "catch_extra_error_codes": ["DirectConnectClientException"]}


@AWSRetry.jittered_backoff(**retry_params)
def describe_connections(client, params):
    return client.describe_connections(**params)


def find_connection_id(client, connection_id=None, connection_name=None):
    params = {}
    if connection_id:
        params["connectionId"] = connection_id
    try:
        response = describe_connections(client, params)
    except (BotoCoreError, ClientError) as e:
        if connection_id:
            msg = f"Failed to describe DirectConnect ID {connection_id}"
        else:
            msg = "Failed to describe DirectConnect connections"
        raise DirectConnectError(
            msg=msg,
            last_traceback=traceback.format_exc(),
            exception=e,
        )

    match = []
    if len(response.get("connections", [])) == 1 and connection_id:
        if response["connections"][0]["connectionState"] != "deleted":
            match.append(response["connections"][0]["connectionId"])

    for conn in response.get("connections", []):
        if connection_name == conn["connectionName"] and conn["connectionState"] != "deleted":
            match.append(conn["connectionId"])

    if len(match) == 1:
        return match[0]
    else:
        raise DirectConnectError(msg="Could not find a valid DirectConnect connection")


def get_connection_state(client, connection_id):
    try:
        response = describe_connections(client, dict(connectionId=connection_id))
        return response["connections"][0]["connectionState"]
    except (BotoCoreError, ClientError, IndexError) as e:
        raise DirectConnectError(
            msg=f"Failed to describe DirectConnect connection {connection_id} state",
            last_traceback=traceback.format_exc(),
            exception=e,
        )


def main():
    argument_spec = dict(connection_id=dict(), name=dict())
    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        mutually_exclusive=[["connection_id", "name"]],
        required_one_of=[["connection_id", "name"]],
    )
    client = module.client("directconnect")

    connection_id = module.params["connection_id"]
    connection_name = module.params["name"]

    changed = False
    connection_state = None
    try:
        connection_id = find_connection_id(client, connection_id, connection_name)
        connection_state = get_connection_state(client, connection_id)
        if connection_state == "ordering":
            client.confirm_connection(connectionId=connection_id)
            changed = True
            connection_state = get_connection_state(client, connection_id)
    except DirectConnectError as e:
        if e.last_traceback:
            module.fail_json(msg=e.msg, exception=e.last_traceback, **camel_dict_to_snake_dict(e.exception.response))
        else:
            module.fail_json(msg=e.msg)

    module.exit_json(changed=changed, connection_state=connection_state)


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The ios_interfaces config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import Facts
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.interfaces import (
    InterfacesTemplate,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.utils.utils import (
    normalize_interface,
)


class Interfaces(ResourceModule):
    """
    The ios_interfaces config class
    """

    def __init__(self, module):
        super(Interfaces, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="interfaces",
            tmplt=InterfacesTemplate(),
        )
        self.parsers = [
            "description",
            "speed",
            "mtu",
            "duplex",
            "template",
            "mac_address",
            "service_policy.input",
            "service_policy.output",
            "service_policy.type_options.access_control",
            "service_policy.type_options.epbr",
            "service_policy.type_options.nwpi",
            "service_policy.type_options.packet_service",
            "service_policy.type_options.service_chain",
            "logging.trunk_status",
            "logging.subif_link_status",
            "logging.status",
            "logging.spanning_tree",
            "logging.nfas_status",
            "logging.bundle_status",
            "logging.link_status",
            "snmp.ifindex",
            "snmp.trap",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = {entry["name"]: entry for entry in self.want}
        haved = {entry["name"]: entry for entry in self.have}

        for each in wantd, haved:
            self.normalize_interface_names(each)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state in ["deleted", "purged"]:
            haved = {k: v for k, v in haved.items() if k in wantd or not wantd}
            wantd = {}

        # remove superfluous config for overridden and deleted
        if self.state in ["overridden", "deleted"]:
            for k, have in haved.items():
                if k not in wantd:
                    self._compare(want={}, have=have)

        if self.state == "purged":
            for k, have in haved.items():
                self.purge(have)
        else:
            for k, want in wantd.items():
                self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Interfaces network resource.
        """
        begin = len(self.commands)
        self.compare(parsers=self.parsers, want=want, have=have)
        want_enabled = want.get("enabled")
        have_enabled = have.get("enabled")
        if want_enabled is not None:
            if want_enabled != have_enabled:
                if want_enabled is True:
                    self.addcmd(want, "enabled", True)
                else:
                    self.addcmd(want, "enabled", False)
        elif not want and self.state == "overridden":
            self.addcmd(have, "enabled", False)
        elif not want and self.state == "deleted":
            if have_enabled:
                self.addcmd(have, "enabled", False)
        if want.get("mode") != have.get("mode"):
            if want.get("mode") == "layer3":
                self.addcmd(want, "mode", True)
            else:
                if want:
                    self.addcmd(want, "mode", False)
                elif have.get("mode"):  # can oly have layer2 as switchport no show cli
                    # handles deleted as want be blank and only
                    self.addcmd(have, "mode", False)
        if len(self.commands) != begin:
            self.commands.insert(begin, self._tmplt.render(want or have, "interface", False))

    def purge(self, have):
        """Handle operation for purged state"""
        self.commands.append(self._tmplt.render(have, "interface", True))

    def normalize_interface_names(self, param):
        if param:
            for k, val in param.items():
                val["name"] = normalize_interface(val["name"])
        return param



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2012, Mark Theunissen <mark.theunissen@gmail.com>
# Sponsored by Four Kitchens http://fourkitchens.com.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: mysql_db
short_description: Add or remove MySQL or MariaDB databases from a remote host
description:
- Add or remove MySQL or MariaDB databases from a remote host.
options:
  name:
    description:
    - Name of the database to add or remove.
    - I(name=all) may only be provided if I(state) is C(dump) or C(import).
    - List of databases is provided with I(state=dump), I(state=present) and I(state=absent).
    - If I(name=all) it works like --all-databases option for mysqldump (Added in 2.0).
    required: true
    type: list
    elements: str
    aliases: [db]
  state:
    description:
    - The database state.
    type: str
    default: present
    choices: ['absent', 'dump', 'import', 'present']
  collation:
    description:
    - Collation mode (sorting). This only applies to new table/databases and
      does not update existing ones, this is a limitation of MySQL.
    type: str
    default: ''
  encoding:
    description:
    - Encoding mode to use, examples include C(utf8) or C(latin1_swedish_ci),
      at creation of database, dump or importation of sql script.
    type: str
    default: ''
  target:
    description:
    - Location, on the remote host, of the dump file to read from or write to.
    - Uncompressed SQL files (C(.sql)) as well as bzip2 (C(.bz2)), gzip (C(.gz)),
      xz (Added in 2.0) and zstd (C(.zst)) (Added in 3.12.0) compressed files are supported.
    type: path
  single_transaction:
    description:
    - Execute the dump in a single transaction.
    type: bool
    default: false
  quick:
    description:
    - Option used for dumping large tables.
    type: bool
    default: true
  ignore_tables:
    description:
    - A list of table names that will be ignored in the dump
      of the form database_name.table_name.
    type: list
    elements: str
    default: []
  hex_blob:
    description:
    - Dump binary columns using hexadecimal notation.
    type: bool
    default: false
    version_added: '0.1.0'
  force:
    description:
    - Continue dump or import even if we get an SQL error.
    - Used only when I(state) is C(dump) or C(import).
    type: bool
    default: false
    version_added: '0.1.0'
  master_data:
    description:
      - Option to dump a master replication server to produce a dump file
        that can be used to set up another server as a slave of the master.
      - C(0) to not include master data.
      - C(1) to generate a 'CHANGE MASTER TO' statement
        required on the slave to start the replication process.
      - C(2) to generate a commented 'CHANGE MASTER TO'.
      - Can be used when I(state=dump).
    type: int
    choices: [0, 1, 2]
    default: 0
    version_added: '0.1.0'
  skip_lock_tables:
    description:
      - Skip locking tables for read. Used when I(state=dump), ignored otherwise.
    type: bool
    default: false
    version_added: '0.1.0'
  dump_extra_args:
    description:
      - Provide additional arguments for mysqldump.
        Used when I(state=dump) only, ignored otherwise.
    type: str
    version_added: '0.1.0'
  use_shell:
    description:
      - Used to prevent C(Broken pipe) errors when the imported I(target) file is compressed.
      - If C(yes), the module will internally execute commands via a shell.
      - Used when I(state=import), ignored otherwise.
    type: bool
    default: false
    version_added: '0.1.0'
  unsafe_login_password:
    description:
      - If C(no), the module will safely use a shell-escaped
        version of the I(login_password) value.
      - It makes sense to use C(yes) only if there are special
        symbols in the value and errors C(Access denied) occur.
      - Used only when I(state) is C(import) or C(dump) and
        I(login_password) is passed, ignored otherwise.
    type: bool
    default: false
    version_added: '0.1.0'
  restrict_config_file:
    description:
      - Read only passed I(config_file).
      - When I(state) is C(dump) or C(import),
        by default the module passes I(config_file) parameter
        using C(--defaults-extra-file) command-line argument to C(mysql/mysqldump) utilities
        under the hood that read named option file in addition to usual option files.
      - If this behavior is undesirable, use C(yes) to read only named option file.
    type: bool
    default: false
    version_added: '0.1.0'
  check_implicit_admin:
    description:
      - Check if mysql allows login as root/nopassword before trying supplied credentials.
      - If success, passed I(login_user)/I(login_password) will be ignored.
    type: bool
    default: false
    version_added: '0.1.0'
  config_overrides_defaults:
    description:
      - If C(yes), connection parameters from I(config_file) will override the default
        values of I(login_host) and I(login_port) parameters.
      - Used when I(stat) is C(present) or C(absent), ignored otherwise.
      - It needs Python 3.5+ as the default interpreter on a target host.
    type: bool
    default: false
    version_added: '0.1.0'
  chdir:
    description:
    - Changes the current working directory.
    - Can be useful, for example, when I(state=import) and a dump file contains relative paths.
    type: path
    version_added: '3.4.0'
  pipefail:
    description:
    - Use C(bash) instead of C(sh) and add C(-o pipefail) to catch errors from the
      mysql_dump command when I(state=dump) and compression is used.
    type: bool
    default: true
    version_added: '3.4.0'
  sql_log_bin:
    description:
      - Whether binary logging should be enabled or disabled for the connection.
    type: bool
    default: true

seealso:
- module: ansible.mysql.mysql_info
- module: ansible.mysql.mysql_variables
- module: ansible.mysql.mysql_user
- module: ansible.mysql.mysql_replication
- name: MySQL command-line client reference
  description: Complete reference of the MySQL command-line client documentation.
  link: https://dev.mysql.com/doc/refman/8.0/en/mysql.html
- name: mysqldump reference
  description: Complete reference of the ``mysqldump`` client utility documentation.
  link: https://dev.mysql.com/doc/refman/8.0/en/mysqldump.html
- name: CREATE DATABASE reference
  description: Complete reference of the CREATE DATABASE command documentation.
  link: https://dev.mysql.com/doc/refman/8.0/en/create-database.html
- name: DROP DATABASE reference
  description: Complete reference of the DROP DATABASE command documentation.
  link: https://dev.mysql.com/doc/refman/8.0/en/drop-database.html
author: "Ansible Core Team"
requirements:
   - mysql (command line binary)
   - mysqldump (command line binary)
notes:
   - Compatible with MariaDB or MySQL.
   - Requires the mysql and mysqldump binaries on the remote host.
   - This module is B(not idempotent) when I(state) is C(import),
     and will import the dump file each time if run more than once.
attributes:
  check_mode:
    support: full
  idempotent:
    support: partial
    details:
      - The module is not idempotent when O(state=import) or when O(state=dump).
extends_documentation_fragment:
- ansible.mysql.mysql
'''

EXAMPLES = r'''
# If you encounter the "Please explicitly state intended protocol" error,
# use the login_unix_socket argument
- name: Create a new database with name 'bobdata'
  ansible.mysql.mysql_db:
    name: bobdata
    state: present
    login_unix_socket: /run/mysqld/mysqld.sock

- name: Create new databases with names 'foo' and 'bar'
  ansible.mysql.mysql_db:
    name:
      - foo
      - bar
    state: present

# Copy database dump file to remote host and restore it to database 'my_db'
- name: Copy database dump file
  copy:
    src: dump.sql.bz2
    dest: /tmp

- name: Restore database
  ansible.mysql.mysql_db:
    name: my_db
    state: import
    target: /tmp/dump.sql.bz2

- name: Restore database ignoring errors
  ansible.mysql.mysql_db:
    name: my_db
    state: import
    target: /tmp/dump.sql.bz2
    force: true

- name: Dump multiple databases
  ansible.mysql.mysql_db:
    state: dump
    name: db_1,db_2
    target: /tmp/dump.sql

- name: Dump multiple databases
  ansible.mysql.mysql_db:
    state: dump
    name:
      - db_1
      - db_2
    target: /tmp/dump.sql

- name: Dump all databases to hostname.sql
  ansible.mysql.mysql_db:
    state: dump
    name: all
    target: /tmp/dump.sql

- name: Dump all databases to hostname.sql including master data
  ansible.mysql.mysql_db:
    state: dump
    name: all
    target: /tmp/dump.sql
    master_data: 1

# Import of sql script with encoding option
- name: >
    Import dump.sql with specific latin1 encoding,
    similar to mysql -u <username> --default-character-set=latin1 -p <password> < dump.sql
  ansible.mysql.mysql_db:
    state: import
    name: all
    encoding: latin1
    target: /tmp/dump.sql

# Dump of database with encoding option
- name: >
    Dump of Databse with specific latin1 encoding,
    similar to mysqldump -u <username> --default-character-set=latin1 -p <password> <database>
  ansible.mysql.mysql_db:
    state: dump
    name: db_1
    encoding: latin1
    target: /tmp/dump.sql

- name: Delete database with name 'bobdata'
  ansible.mysql.mysql_db:
    name: bobdata
    state: absent

- name: Make sure there is neither a database with name 'foo', nor one with name 'bar'
  ansible.mysql.mysql_db:
    name:
      - foo
      - bar
    state: absent

# Dump database with argument not directly supported by this module
# using dump_extra_args parameter
- name: Dump databases without including triggers
  ansible.mysql.mysql_db:
    state: dump
    name: foo
    target: /tmp/dump.sql
    dump_extra_args: --skip-triggers

- name: Try to create database as root/nopassword first. If not allowed, pass the credentials
  ansible.mysql.mysql_db:
    check_implicit_admin: true
    login_user: bob
    login_password: 123456
    name: bobdata
    state: present

- name: Dump a database with compression and catch errors from mysqldump with bash pipefail
  ansible.mysql.mysql_db:
    state: dump
    name: foo
    target: /tmp/dump.sql.gz
    pipefail: true
'''

RETURN = r'''
db:
  description: Database names in string format delimited by white space.
  returned: always
  type: str
  sample: "foo bar"
db_list:
  description: List of database names.
  returned: always
  type: list
  sample: ["foo", "bar"]
executed_commands:
  description: List of commands which tried to run.
  returned: if executed
  type: list
  sample: ["CREATE DATABASE acme"]
  version_added: '0.1.0'
'''

import os
import subprocess
import traceback
import shlex

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.mysql.plugins.module_utils.database import mysql_quote_identifier
from ansible_collections.ansible.mysql.plugins.module_utils.mysql import (
    mysql_connect,
    mysql_driver,
    mysql_driver_fail_msg,
    mysql_common_argument_spec,
    get_server_implementation,
    get_server_version,
)
from ansible_collections.ansible.mysql.plugins.module_utils.version import LooseVersion
from ansible.module_utils.common.text.converters import to_native

executed_commands = []

# ===========================================
# MySQL module specific support methods.
#


def db_exists(cursor, db):
    res = 0
    for each_db in db:
        res += cursor.execute("SELECT SCHEMA_NAME FROM information_schema.SCHEMATA WHERE SCHEMA_NAME = %s", (each_db,))
    return res == len(db)


def db_delete(cursor, db):
    if not db:
        return False
    for each_db in db:
        query = "DROP DATABASE %s" % mysql_quote_identifier(each_db, 'database')
        executed_commands.append(query)
        cursor.execute(query)
    return True


def db_dump(module, host, user, password, db_name, target, all_databases, port,
            config_file, server_implementation, server_version, socket=None,
            ssl_cert=None, ssl_key=None, ssl_ca=None,
            single_transaction=None, quick=None, ignore_tables=None, hex_blob=None,
            encoding=None, force=False, master_data=0, skip_lock_tables=False,
            dump_extra_args=None, unsafe_password=False, restrict_config_file=False,
            check_implicit_admin=False, pipefail=False):

    cmd_str = 'mysqldump'
    if server_implementation == 'mariadb' and LooseVersion(server_version) >= LooseVersion("10.4.6"):
        cmd_str = 'mariadb-dump'
    try:
        cmd = [module.get_bin_path(cmd_str, True)]
    except Exception as e:
        return 1, "", "Error determining dump command: %s" % str(e)

    # If defined, mysqldump demands --defaults-extra-file be the first option
    if config_file:
        if restrict_config_file:
            cmd.append("--defaults-file=%s" % shlex.quote(config_file))
        else:
            cmd.append("--defaults-extra-file=%s" % shlex.quote(config_file))

    if check_implicit_admin:
        cmd.append("--user=root --password=''")
    else:
        if user is not None:
            cmd.append("--user=%s" % shlex.quote(user))

        if password is not None:
            if not unsafe_password:
                cmd.append("--password=%s" % shlex.quote(password))
            else:
                cmd.append("--password=%s" % password)

    if ssl_cert is not None:
        cmd.append("--ssl-cert=%s" % shlex.quote(ssl_cert))
    if ssl_key is not None:
        cmd.append("--ssl-key=%s" % shlex.quote(ssl_key))
    if ssl_ca is not None:
        cmd.append("--ssl-ca=%s" % shlex.quote(ssl_ca))
    if force:
        cmd.append("--force")
    if socket is not None:
        cmd.append("--socket=%s" % shlex.quote(socket))
    else:
        cmd.append("--host=%s --port=%i" % (shlex.quote(host), port))

    if all_databases:
        cmd.append("--all-databases")
    elif len(db_name) > 1:
        cmd.append("--databases {0}".format(' '.join(db_name)))
    else:
        cmd.append("%s" % shlex.quote(' '.join(db_name)))

    if skip_lock_tables:
        cmd.append("--skip-lock-tables")
    if (encoding is not None) and (encoding != ""):
        cmd.append("--default-character-set=%s" % shlex.quote(encoding))
    if single_transaction:
        cmd.append("--single-transaction=true")
    if quick:
        cmd.append("--quick")
    if ignore_tables:
        for an_ignored_table in ignore_tables:
            cmd.append("--ignore-table={0}".format(an_ignored_table))
    if hex_blob:
        cmd.append("--hex-blob")
    if master_data:
        if (server_implementation == 'mysql' and
                LooseVersion(server_version) >= LooseVersion("8.2.0")):
            cmd.append("--source-data=%s" % master_data)
        else:
            cmd.append("--master-data=%s" % master_data)
    if dump_extra_args is not None:
        cmd.append(dump_extra_args)

    path = None
    if os.path.splitext(target)[-1] == '.gz':
        path = module.get_bin_path('gzip', True)
    elif os.path.splitext(target)[-1] == '.bz2':
        path = module.get_bin_path('bzip2', True)
    elif os.path.splitext(target)[-1] == '.xz':
        path = module.get_bin_path('xz', True)
    elif os.path.splitext(target)[-1] == '.zst':
        path = module.get_bin_path('zstd', True)

    cmd = ' '.join(cmd)

    if path:
        cmd = '%s | %s > %s' % (cmd, path, shlex.quote(target))
        if pipefail:
            cmd = 'set -o pipefail && ' + cmd
    else:
        cmd += " > %s" % shlex.quote(target)

    executed_commands.append(cmd)

    if pipefail:
        rc, stdout, stderr = module.run_command(cmd, use_unsafe_shell=True, executable='bash')
    else:
        rc, stdout, stderr = module.run_command(cmd, use_unsafe_shell=True)

    return rc, stdout, stderr


def db_import(module, host, user, password, db_name, target, all_databases, port, config_file,
              server_implementation, server_version, socket=None, ssl_cert=None, ssl_key=None, ssl_ca=None,
              encoding=None, force=False,
              use_shell=False, unsafe_password=False, restrict_config_file=False,
              check_implicit_admin=False):
    if not os.path.exists(target):
        return module.fail_json(msg="target %s does not exist on the host" % target)

    cmd_str = 'mysql'
    if server_implementation == 'mariadb' and LooseVersion(server_version) >= LooseVersion("10.4.6"):
        cmd_str = 'mariadb'
    try:
        cmd = [module.get_bin_path(cmd_str, True)]
    except Exception as e:
        return 1, "", "Error determining mysql/mariadb command: %s" % str(e)

    # --defaults-file must go first, or errors out
    if config_file:
        if restrict_config_file:
            cmd.append("--defaults-file=%s" % shlex.quote(config_file))
        else:
            cmd.append("--defaults-extra-file=%s" % shlex.quote(config_file))

    if check_implicit_admin:
        cmd.append("--user=root --password=''")
    else:
        if user:
            cmd.append("--user=%s" % shlex.quote(user))

        if password:
            if not unsafe_password:
                cmd.append("--password=%s" % shlex.quote(password))
            else:
                cmd.append("--password=%s" % password)

    if ssl_cert is not None:
        cmd.append("--ssl-cert=%s" % shlex.quote(ssl_cert))
    if ssl_key is not None:
        cmd.append("--ssl-key=%s" % shlex.quote(ssl_key))
    if ssl_ca is not None:
        cmd.append("--ssl-ca=%s" % shlex.quote(ssl_ca))
    if force:
        cmd.append("-f")
    if socket is not None:
        cmd.append("--socket=%s" % shlex.quote(socket))
    else:
        cmd.append("--host=%s" % shlex.quote(host))
        cmd.append("--port=%i" % port)
    if (encoding is not None) and (encoding != ""):
        cmd.append("--default-character-set=%s" % shlex.quote(encoding))
    if not all_databases:
        cmd.append("--one-database")
        cmd.append(shlex.quote(''.join(db_name)))

    comp_prog_path = None
    if os.path.splitext(target)[-1] == '.gz':
        comp_prog_path = module.get_bin_path('gzip', required=True)
    elif os.path.splitext(target)[-1] == '.bz2':
        comp_prog_path = module.get_bin_path('bzip2', required=True)
    elif os.path.splitext(target)[-1] == '.xz':
        comp_prog_path = module.get_bin_path('xz', required=True)
    elif os.path.splitext(target)[-1] == '.zst':
        comp_prog_path = module.get_bin_path('zstd', required=True)
    if comp_prog_path:
        # The line below is for returned data only:
        executed_commands.append('%s -dc %s | %s' % (comp_prog_path, target, cmd))

        if not use_shell:
            p1 = subprocess.Popen([comp_prog_path, '-dc', target], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            p2 = subprocess.Popen(cmd, stdin=p1.stdout, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            (stdout2, stderr2) = p2.communicate()
            p1.stdout.close()
            p1.wait()

            if p1.returncode != 0:
                stderr1 = p1.stderr.read()
                return p1.returncode, '', stderr1
            else:
                return p2.returncode, stdout2, stderr2
        else:
            # Used to prevent 'Broken pipe' errors that
            # occasionaly occur when target files are compressed.
            # FYI: passing the `shell=True` argument to p2 = subprocess.Popen()
            # doesn't solve the problem.
            cmd = " ".join(cmd)
            cmd = "%s -dc %s | %s" % (comp_prog_path, shlex.quote(target), cmd)
            rc, stdout, stderr = module.run_command(cmd, use_unsafe_shell=True)
            return rc, stdout, stderr

    else:
        cmd = ' '.join(cmd)
        cmd += " < %s" % shlex.quote(target)
        executed_commands.append(cmd)
        rc, stdout, stderr = module.run_command(cmd, use_unsafe_shell=True)
        return rc, stdout, stderr


def db_create(cursor, db, encoding, collation):
    if not db:
        return False
    query_params = dict(enc=encoding, collate=collation)
    res = 0
    for each_db in db:
        # Escape '%' since mysql cursor.execute() uses a format string
        query = ['CREATE DATABASE %s' % mysql_quote_identifier(each_db, 'database').replace('%', '%%')]
        if encoding:
            query.append("CHARACTER SET %(enc)s")
        if collation:
            query.append("COLLATE %(collate)s")
        query = ' '.join(query)
        res += cursor.execute(query, query_params)
        try:
            executed_commands.append(cursor.mogrify(query, query_params))
        except AttributeError:
            executed_commands.append(cursor._executed)
        except Exception:
            executed_commands.append(query)
    return res > 0


# ===========================================
# Module execution.
#


def main():
    argument_spec = mysql_common_argument_spec()
    argument_spec.update(
        name=dict(type='list', elements='str', required=True, aliases=['db']),
        encoding=dict(type='str', default=''),
        collation=dict(type='str', default=''),
        target=dict(type='path'),
        state=dict(type='str', default='present', choices=['absent', 'dump', 'import', 'present']),
        single_transaction=dict(type='bool', default=False),
        quick=dict(type='bool', default=True),
        ignore_tables=dict(type='list', elements='str', default=[]),
        hex_blob=dict(default=False, type='bool'),
        force=dict(type='bool', default=False),
        master_data=dict(type='int', default=0, choices=[0, 1, 2]),
        skip_lock_tables=dict(type='bool', default=False),
        dump_extra_args=dict(type='str'),
        use_shell=dict(type='bool', default=False),
        unsafe_login_password=dict(type='bool', default=False, no_log=True),
        restrict_config_file=dict(type='bool', default=False),
        check_implicit_admin=dict(type='bool', default=False),
        config_overrides_defaults=dict(type='bool', default=False),
        chdir=dict(type='path'),
        pipefail=dict(type='bool', default=True),
        sql_log_bin=dict(type='bool', default=True),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    if mysql_driver is None:
        module.fail_json(msg=mysql_driver_fail_msg)

    db = module.params["name"]
    if not db:
        module.exit_json(changed=False, db=db, db_list=[])
    db = [each_db.strip() for each_db in db]

    encoding = module.params["encoding"]
    collation = module.params["collation"]
    state = module.params["state"]
    target = module.params["target"]
    socket = module.params["login_unix_socket"]
    login_port = module.params["login_port"]
    if login_port < 0 or login_port > 65535:
        module.fail_json(msg="login_port must be a valid unix port number (0-65535)")
    ssl_cert = module.params["client_cert"]
    ssl_key = module.params["client_key"]
    ssl_ca = module.params["ca_cert"]
    check_hostname = module.params["check_hostname"]
    connect_timeout = module.params['connect_timeout']
    config_file = module.params['config_file']
    login_password = module.params["login_password"]
    unsafe_login_password = module.params["unsafe_login_password"]
    login_user = module.params["login_user"]
    login_host = module.params["login_host"]
    ignore_tables = module.params["ignore_tables"]
    for a_table in ignore_tables:
        if a_table == "":
            module.fail_json(msg="Name of ignored table cannot be empty")
    single_transaction = module.params["single_transaction"]
    quick = module.params["quick"]
    hex_blob = module.params["hex_blob"]
    force = module.params["force"]
    master_data = module.params["master_data"]
    skip_lock_tables = module.params["skip_lock_tables"]
    dump_extra_args = module.params["dump_extra_args"]
    use_shell = module.params["use_shell"]
    restrict_config_file = module.params["restrict_config_file"]
    check_implicit_admin = module.params['check_implicit_admin']
    config_overrides_defaults = module.params['config_overrides_defaults']
    chdir = module.params['chdir']
    pipefail = module.params['pipefail']
    sql_log_bin = module.params["sql_log_bin"]

    if chdir:
        try:
            os.chdir(chdir)
        except Exception as e:
            module.fail_json("Cannot change the current directory to %s: %s" % (chdir, e))

    if len(db) > 1 and state == 'import':
        module.fail_json(msg="Multiple databases are not supported with state=import")
    db_name = ' '.join(db)

    all_databases = False
    if state in ['dump', 'import']:
        if target is None:
            module.fail_json(msg="with state=%s target is required" % state)
        if db == ['all']:
            all_databases = True
    else:
        if db == ['all']:
            module.fail_json(msg="name is not allowed to equal 'all' unless state equals import, or dump.")
    try:
        cursor = None
        if check_implicit_admin:
            try:
                cursor, db_conn = mysql_connect(module, 'root', '', config_file, ssl_cert, ssl_key, ssl_ca,
                                                connect_timeout=connect_timeout, check_hostname=check_hostname,
                                                config_overrides_defaults=config_overrides_defaults)
            except Exception as e:
                check_implicit_admin = False
                pass

        if not cursor:
            cursor, db_conn = mysql_connect(module, login_user, login_password, config_file, ssl_cert, ssl_key, ssl_ca,
                                            connect_timeout=connect_timeout, config_overrides_defaults=config_overrides_defaults,
                                            check_hostname=check_hostname)
    except Exception as e:
        if os.path.exists(config_file):
            module.fail_json(msg="unable to connect to database, check login_user and login_password are correct or %s has the credentials. "
                                 "Exception message: %s" % (config_file, to_native(e)))
        else:
            module.fail_json(msg="unable to find %s. Exception message: %s" % (config_file, to_native(e)))

    if state in ['absent', 'present'] and not sql_log_bin:
        cursor.execute("SET SQL_LOG_BIN=0;")

    server_implementation = get_server_implementation(cursor)
    server_version = get_server_version(cursor)

    changed = False
    if not os.path.exists(config_file):
        config_file = None

    existence_list = []
    non_existence_list = []

    if not all_databases:
        for each_database in db:
            if db_exists(cursor, [each_database]):
                existence_list.append(each_database)
            else:
                non_existence_list.append(each_database)

    if state == "absent":
        if module.check_mode:
            module.exit_json(changed=bool(existence_list), db=db_name, db_list=db)
        try:
            changed = db_delete(cursor, existence_list)
        except Exception as e:
            module.fail_json(msg="error deleting database: %s" % to_native(e))
        module.exit_json(changed=changed, db=db_name, db_list=db, executed_commands=executed_commands)
    elif state == "present":
        if module.check_mode:
            module.exit_json(changed=bool(non_existence_list), db=db_name, db_list=db)
        changed = False
        if non_existence_list:
            try:
                changed = db_create(cursor, non_existence_list, encoding, collation)
            except Exception as e:
                module.fail_json(msg="error creating database: %s" % to_native(e),
                                 exception=traceback.format_exc())
        module.exit_json(changed=changed, db=db_name, db_list=db, executed_commands=executed_commands)
    elif state == "dump":
        if non_existence_list and not all_databases:
            module.fail_json(msg="Cannot dump database(s) %r - not found" % (', '.join(non_existence_list)))
        if module.check_mode:
            module.exit_json(changed=True, db=db_name, db_list=db)
        rc, stdout, stderr = db_dump(module, login_host, login_user,
                                     login_password, db, target, all_databases,
                                     login_port, config_file, server_implementation, server_version,
                                     socket, ssl_cert, ssl_key,
                                     ssl_ca, single_transaction, quick, ignore_tables,
                                     hex_blob, encoding, force, master_data, skip_lock_tables,
                                     dump_extra_args, unsafe_login_password, restrict_config_file,
                                     check_implicit_admin, pipefail)
        if rc != 0:
            module.fail_json(msg="%s" % stderr)
        module.exit_json(changed=True, db=db_name, db_list=db, msg=stdout,
                         executed_commands=executed_commands)
    elif state == "import":
        if module.check_mode:
            module.exit_json(changed=True, db=db_name, db_list=db)
        if non_existence_list and not all_databases:
            try:
                db_create(cursor, non_existence_list, encoding, collation)
            except Exception as e:
                module.fail_json(msg="error creating database: %s" % to_native(e),
                                 exception=traceback.format_exc())
        rc, stdout, stderr = db_import(module, login_host, login_user,
                                       login_password, db, target,
                                       all_databases,
                                       login_port, config_file, server_implementation,
                                       server_version, socket, ssl_cert, ssl_key, ssl_ca,
                                       encoding, force, use_shell, unsafe_login_password,
                                       restrict_config_file, check_implicit_admin)
        if rc != 0:
            module.fail_json(msg="%s" % stderr)
        module.exit_json(changed=True, db=db_name, db_list=db, msg=stdout,
                         executed_commands=executed_commands)


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2019 Gregory Thiemonge <gregory.thiemonge@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: gandi_livedns
author:
  - Gregory Thiemonge (@gthiemonge)
version_added: "2.3.0"
short_description: Manage Gandi LiveDNS records
description:
  - 'Manages DNS records by the Gandi LiveDNS API, see the docs: U(https://doc.livedns.gandi.net/).'
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
    version_added: 13.0.0
options:
  personal_access_token:
    description:
      - Scoped API token.
      - One of O(personal_access_token) and O(api_key) must be specified.
    type: str
    version_added: 9.0.0
  api_key:
    description:
      - Account API token.
      - Note that these type of keys are deprecated and might stop working at some point. Use personal access tokens instead.
      - One of O(personal_access_token) and O(api_key) must be specified.
    type: str
  record:
    description:
      - Record to add.
    type: str
    required: true
  state:
    description:
      - Whether the record(s) should exist or not.
    type: str
    choices: [absent, present]
    default: present
  ttl:
    description:
      - The TTL to give the new record.
      - Required when O(state=present).
    type: int
  type:
    description:
      - The type of DNS record to create.
    type: str
    required: true
  values:
    description:
      - The record values.
      - Required when O(state=present).
    type: list
    elements: str
  domain:
    description:
      - The name of the Domain to work with (for example, V(example.com)).
    required: true
    type: str
"""

EXAMPLES = r"""
- name: Create a test A record to point to 127.0.0.1 in the my.com domain
  community.general.gandi_livedns:
    domain: my.com
    record: test
    type: A
    values:
      - 127.0.0.1
    ttl: 7200
    personal_access_token: dummytoken
  register: record

- name: Create a mail CNAME record to www.my.com domain
  community.general.gandi_livedns:
    domain: my.com
    type: CNAME
    record: mail
    values:
      - www
    ttl: 7200
    personal_access_token: dummytoken
    state: present

- name: Change its TTL
  community.general.gandi_livedns:
    domain: my.com
    type: CNAME
    record: mail
    values:
      - www
    ttl: 10800
    personal_access_token: dummytoken
    state: present

- name: Delete the record
  community.general.gandi_livedns:
    domain: my.com
    type: CNAME
    record: mail
    personal_access_token: dummytoken
    state: absent

- name: Use a (deprecated) API Key
  community.general.gandi_livedns:
    domain: my.com
    record: test
    type: A
    values:
      - 127.0.0.1
    ttl: 7200
    api_key: dummyapikey
"""

RETURN = r"""
record:
  description: A dictionary containing the record data.
  returned: success, except on record deletion
  type: dict
  contains:
    values:
      description: The record content (details depend on record type).
      returned: success
      type: list
      elements: str
      sample:
        - 192.0.2.91
        - 192.0.2.92
    record:
      description: The record name.
      returned: success
      type: str
      sample: www
    ttl:
      description: The time-to-live for the record.
      returned: success
      type: int
      sample: 300
    type:
      description: The record type.
      returned: success
      type: str
      sample: A
    domain:
      description: The domain associated with the record.
      returned: success
      type: str
      sample: my.com
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._gandi_livedns_api import GandiLiveDNSAPI


def main():
    module = AnsibleModule(
        argument_spec=dict(
            api_key=dict(type="str", no_log=True),
            personal_access_token=dict(type="str", no_log=True),
            record=dict(type="str", required=True),
            state=dict(type="str", default="present", choices=["absent", "present"]),
            ttl=dict(type="int"),
            type=dict(type="str", required=True),
            values=dict(type="list", elements="str"),
            domain=dict(type="str", required=True),
        ),
        supports_check_mode=True,
        required_if=[
            ("state", "present", ["values", "ttl"]),
        ],
        mutually_exclusive=[
            ("api_key", "personal_access_token"),
        ],
        required_one_of=[
            ("api_key", "personal_access_token"),
        ],
    )

    gandi_api = GandiLiveDNSAPI(module)

    domain = module.params["domain"]

    if module.params["state"] == "present":
        before, ret, changed = gandi_api.ensure_dns_record(
            module.params["record"],
            module.params["type"],
            module.params["ttl"],
            module.params["values"],
            domain,
        )
    else:
        before, ret, changed = gandi_api.delete_dns_record(
            module.params["record"], module.params["type"], module.params["values"], domain
        )

    result = dict(
        changed=changed,
    )
    if ret:
        result["record"] = gandi_api.build_result(ret, domain)

    if module._diff:
        result["diff"] = {
            "before": gandi_api.build_result(before, domain) or {},
            "after": result.get("record") or {},
        }

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_where_used
short_description: Searches for usage of the target object in other objects and rules.
description:
  - Searches for usage of the target object in other objects and rules.
  - All operations are performed over Web Services API.
  - Available from R80 management version.
version_added: "5.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
  dereference_group_members:
    description:
      - Indicates whether to dereference "members" field by details level for every object in reply.
      - Available from R80.10 management version.
    type: bool
  show_membership:
    description:
      - Indicates whether to calculate and show "groups" field for every object in reply.
      - Available from R80.10 management version.
    type: bool
  async_response:
    description:
      - Run command in asynchronous mode and return task UID. Use show-task command to check the progress of the task.
      - Available from R81.20 management version.
    type: bool
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
      - Available from R81 management version.
    type: list
    elements: str
  indirect:
    description:
      - Search for indirect usage.
    type: bool
  indirect_max_depth:
    description:
      - Maximum nesting level during indirect usage search.
    type: int
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: where-used
  cp_mgmt_where_used:
    name: Host 1
"""

RETURN = """
cp_mgmt_where_used:
  description: The checkpoint where-used output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        name=dict(type='str'),
        dereference_group_members=dict(type='bool'),
        show_membership=dict(type='bool'),
        async_response=dict(type='bool'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        domains_to_process=dict(type='list', elements='str'),
        indirect=dict(type='bool'),
        indirect_max_depth=dict(type='int')
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "where-used"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: kinesis_stream
version_added: 1.0.0
short_description: Manage a Kinesis Stream.
description:
  - Create or Delete a Kinesis Stream.
  - Update the retention period of a Kinesis Stream.
  - Update Tags on a Kinesis Stream.
  - Enable/disable server side encryption on a Kinesis Stream.
author:
  - Allen Sanabria (@linuxdynasty)
options:
  name:
    description:
      - The name of the Kinesis Stream you are managing.
    required: true
    type: str
  shards:
    description:
      - The number of shards you want to have with this stream.
      - This is required when I(state=present)
    type: int
  retention_period:
    description:
      - The length of time (in hours) data records are accessible after they are added to
        the stream.
      - The default retention period is 24 hours and can not be less than 24 hours.
      - The maximum retention period is 168 hours.
      - The retention period can be modified during any point in time.
    type: int
  state:
    description:
      - Create or Delete the Kinesis Stream.
    default: present
    choices: [ 'present', 'absent' ]
    type: str
  wait:
    description:
      - Wait for operation to complete before returning.
    default: true
    type: bool
  wait_timeout:
    description:
      - How many seconds to wait for an operation to complete before timing out.
    default: 300
    type: int
  tags:
    description:
      - "A dictionary of resource tags of the form: C({ tag1: value1, tag2: value2 })."
    aliases: [ "resource_tags" ]
    type: dict
  encryption_state:
    description:
      - Enable or Disable encryption on the Kinesis Stream.
    choices: [ 'enabled', 'disabled' ]
    type: str
  encryption_type:
    description:
      - The type of encryption.
      - Defaults to C(KMS)
    choices: ['KMS', 'NONE']
    type: str
  key_id:
    description:
      - The GUID or alias for the KMS key.
    type: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Basic creation example:
- name: Set up Kinesis Stream with 10 shards and wait for the stream to become ACTIVE
  community.aws.kinesis_stream:
    name: test-stream
    shards: 10
    wait: true
    wait_timeout: 600
  register: test_stream

# Basic creation example with tags:
- name: Set up Kinesis Stream with 10 shards, tag the environment, and wait for the stream to become ACTIVE
  community.aws.kinesis_stream:
    name: test-stream
    shards: 10
    tags:
      Env: development
    wait: true
    wait_timeout: 600
  register: test_stream

# Basic creation example with tags and increase the retention period from the default 24 hours to 48 hours:
- name: Set up Kinesis Stream with 10 shards, tag the environment, increase the retention period and wait for the stream to become ACTIVE
  community.aws.kinesis_stream:
    name: test-stream
    retention_period: 48
    shards: 10
    tags:
      Env: development
    wait: true
    wait_timeout: 600
  register: test_stream

# Basic delete example:
- name: Delete Kinesis Stream test-stream and wait for it to finish deleting.
  community.aws.kinesis_stream:
    name: test-stream
    state: absent
    wait: true
    wait_timeout: 600
  register: test_stream

# Basic enable encryption example:
- name: Encrypt Kinesis Stream test-stream.
  community.aws.kinesis_stream:
    name: test-stream
    state: present
    shards: 1
    encryption_state: enabled
    encryption_type: KMS
    key_id: alias/aws/kinesis
    wait: true
    wait_timeout: 600
  register: test_stream

# Basic disable encryption example:
- name: Encrypt Kinesis Stream test-stream.
  community.aws.kinesis_stream:
    name: test-stream
    state: present
    shards: 1
    encryption_state: disabled
    encryption_type: KMS
    key_id: alias/aws/kinesis
    wait: true
    wait_timeout: 600
  register: test_stream
"""

RETURN = r"""
stream_name:
  description: The name of the Kinesis Stream.
  returned: when state == present.
  type: str
  sample: "test-stream"
stream_arn:
  description: The amazon resource identifier
  returned: when state == present.
  type: str
  sample: "arn:aws:kinesis:east-side:123456789:stream/test-stream"
stream_status:
  description: The current state of the Kinesis Stream.
  returned: when state == present.
  type: str
  sample: "ACTIVE"
retention_period_hours:
  description: Number of hours messages will be kept for a Kinesis Stream.
  returned: when state == present.
  type: int
  sample: 24
tags:
  description: Dictionary containing all the tags associated with the Kinesis stream.
  returned: when state == present.
  type: dict
  sample: {
      "Name": "Splunk",
      "Env": "development"
  }
"""

import time

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils._text import to_native
from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def get_tags(client, stream_name):
    """Retrieve the tags for a Kinesis Stream.
    Args:
        client (botocore.client.EC2): Boto3 client.
        stream_name (str): Name of the Kinesis stream.

    Basic Usage:
        >>> client = boto3.client('kinesis')
        >>> stream_name = 'test-stream'
        >> get_tags(client, stream_name)

    Returns:
        Tuple (bool, str, dict)
    """
    err_msg = ""
    success = False
    params = {
        "StreamName": stream_name,
    }
    results = dict()
    try:
        results = client.list_tags_for_stream(**params)["Tags"]
        success = True
    except botocore.exceptions.ClientError as e:
        err_msg = to_native(e)

    return success, err_msg, boto3_tag_list_to_ansible_dict(results)


def find_stream(client, stream_name):
    """Retrieve a Kinesis Stream.
    Args:
        client (botocore.client.EC2): Boto3 client.
        stream_name (str): Name of the Kinesis stream.

    Basic Usage:
        >>> client = boto3.client('kinesis')
        >>> stream_name = 'test-stream'

    Returns:
        Tuple (bool, str, dict)
    """
    err_msg = ""
    success = False
    params = {
        "StreamName": stream_name,
    }
    results = dict()
    has_more_shards = True
    shards = list()
    try:
        while has_more_shards:
            results = client.describe_stream(**params)["StreamDescription"]
            shards.extend(results.pop("Shards"))
            has_more_shards = results["HasMoreShards"]
            if has_more_shards:
                params["ExclusiveStartShardId"] = shards[-1]["ShardId"]
        results["Shards"] = shards
        num_closed_shards = len([s for s in shards if "EndingSequenceNumber" in s["SequenceNumberRange"]])
        results["OpenShardsCount"] = len(shards) - num_closed_shards
        results["ClosedShardsCount"] = num_closed_shards
        results["ShardsCount"] = len(shards)
        success = True
    except botocore.exceptions.ClientError as e:
        err_msg = to_native(e)

    return success, err_msg, results


def wait_for_status(client, stream_name, status, wait_timeout=300, check_mode=False):
    """Wait for the status to change for a Kinesis Stream.
    Args:
        client (botocore.client.EC2): Boto3 client
        stream_name (str): The name of the kinesis stream.
        status (str): The status to wait for.
            examples. status=available, status=deleted

    Kwargs:
        wait_timeout (int): Number of seconds to wait, until this timeout is reached.
        check_mode (bool): This will pass DryRun as one of the parameters to the aws api.
            default=False

    Basic Usage:
        >>> client = boto3.client('kinesis')
        >>> stream_name = 'test-stream'
        >>> wait_for_status(client, stream_name, 'ACTIVE', 300)

    Returns:
        Tuple (bool, str, dict)
    """
    polling_increment_secs = 5
    wait_timeout = time.time() + wait_timeout
    status_achieved = False
    stream = dict()
    err_msg = ""

    while wait_timeout > time.time():
        try:
            find_success, find_msg, stream = find_stream(client, stream_name)
            if check_mode:
                status_achieved = True
                break

            if status != "DELETING":
                if find_success and stream:
                    if stream.get("StreamStatus") == status:
                        status_achieved = True
                        break

            else:
                if not find_success:
                    status_achieved = True
                    break

        except botocore.exceptions.ClientError as e:
            err_msg = to_native(e)

        time.sleep(polling_increment_secs)

    if not status_achieved:
        err_msg = "Wait time out reached, while waiting for results"
    else:
        err_msg = f"Status {status} achieved successfully"

    return status_achieved, err_msg, stream


def tags_action(client, stream_name, tags, action="create", check_mode=False):
    """Create or delete multiple tags from a Kinesis Stream.
    Args:
        client (botocore.client.EC2): Boto3 client.
        resource_id (str): The Amazon resource id.
        tags (list): List of dictionaries.
            examples.. [{Name: "", Values: [""]}]

    Kwargs:
        action (str): The action to perform.
            valid actions == create and delete
            default=create
        check_mode (bool): This will pass DryRun as one of the parameters to the aws api.
            default=False

    Basic Usage:
        >>> client = boto3.client('ec2')
        >>> resource_id = 'pcx-123345678'
        >>> tags = {'env': 'development'}
        >>> update_tags(client, resource_id, tags)
        [True, '']

    Returns:
        List (bool, str)
    """
    success = False
    err_msg = ""
    params = {"StreamName": stream_name}
    try:
        if not check_mode:
            if action == "create":
                params["Tags"] = tags
                client.add_tags_to_stream(**params)
                success = True
            elif action == "delete":
                params["TagKeys"] = tags
                client.remove_tags_from_stream(**params)
                success = True
            else:
                err_msg = f"Invalid action {action}"
        else:
            if action == "create":
                success = True
            elif action == "delete":
                success = True
            else:
                err_msg = f"Invalid action {action}"

    except botocore.exceptions.ClientError as e:
        err_msg = to_native(e)

    return success, err_msg


def update_tags(client, stream_name, tags, check_mode=False):
    """Update tags for an amazon resource.
    Args:
        resource_id (str): The Amazon resource id.
        tags (dict): Dictionary of tags you want applied to the Kinesis stream.

    Kwargs:
        check_mode (bool): This will pass DryRun as one of the parameters to the aws api.
            default=False

    Basic Usage:
        >>> client = boto3.client('ec2')
        >>> stream_name = 'test-stream'
        >>> tags = {'env': 'development'}
        >>> update_tags(client, stream_name, tags)
        [True, '']

    Return:
        Tuple (bool, str)
    """
    success = False
    changed = False
    err_msg = ""
    tag_success, tag_msg, current_tags = get_tags(client, stream_name)

    tags_to_set, tags_to_delete = compare_aws_tags(
        current_tags,
        tags,
        purge_tags=True,
    )
    if tags_to_delete:
        delete_success, delete_msg = tags_action(
            client, stream_name, tags_to_delete, action="delete", check_mode=check_mode
        )
        if not delete_success:
            return delete_success, changed, delete_msg

    if tags_to_set:
        create_success, create_msg = tags_action(
            client, stream_name, tags_to_set, action="create", check_mode=check_mode
        )
        if create_success:
            changed = True
        return create_success, changed, create_msg

    return success, changed, err_msg


def stream_action(client, stream_name, shard_count=1, action="create", timeout=300, check_mode=False):
    """Create or Delete an Amazon Kinesis Stream.
    Args:
        client (botocore.client.EC2): Boto3 client.
        stream_name (str): The name of the kinesis stream.

    Kwargs:
        shard_count (int): Number of shards this stream will use.
        action (str): The action to perform.
            valid actions == create and delete
            default=create
        check_mode (bool): This will pass DryRun as one of the parameters to the aws api.
            default=False

    Basic Usage:
        >>> client = boto3.client('kinesis')
        >>> stream_name = 'test-stream'
        >>> shard_count = 20
        >>> stream_action(client, stream_name, shard_count, action='create')

    Returns:
        List (bool, str)
    """
    success = False
    err_msg = ""
    params = {"StreamName": stream_name}
    try:
        if not check_mode:
            if action == "create":
                params["ShardCount"] = shard_count
                client.create_stream(**params)
                success = True
            elif action == "delete":
                client.delete_stream(**params)
                success = True
            else:
                err_msg = f"Invalid action {action}"
        else:
            if action == "create":
                success = True
            elif action == "delete":
                success = True
            else:
                err_msg = f"Invalid action {action}"

    except botocore.exceptions.ClientError as e:
        err_msg = to_native(e)

    return success, err_msg


def stream_encryption_action(
    client, stream_name, action="start_encryption", encryption_type="", key_id="", timeout=300, check_mode=False
):
    """Create, Encrypt or Delete an Amazon Kinesis Stream.
    Args:
        client (botocore.client.EC2): Boto3 client.
        stream_name (str): The name of the kinesis stream.

    Kwargs:
        shard_count (int): Number of shards this stream will use.
        action (str): The action to perform.
            valid actions == create and delete
            default=create
        encryption_type (str): NONE or KMS
        key_id (str): The GUID or alias for the KMS key
        check_mode (bool): This will pass DryRun as one of the parameters to the aws api.
            default=False

    Basic Usage:
        >>> client = boto3.client('kinesis')
        >>> stream_name = 'test-stream'
        >>> shard_count = 20
        >>> stream_action(client, stream_name, shard_count, action='create', encryption_type='KMS',key_id='alias/aws')

    Returns:
        List (bool, str)
    """
    success = False
    err_msg = ""
    params = {"StreamName": stream_name}
    try:
        if not check_mode:
            if action == "start_encryption":
                params["EncryptionType"] = encryption_type
                params["KeyId"] = key_id
                client.start_stream_encryption(**params)
                success = True
            elif action == "stop_encryption":
                params["EncryptionType"] = encryption_type
                params["KeyId"] = key_id
                client.stop_stream_encryption(**params)
                success = True
            else:
                err_msg = f"Invalid encryption action {action}"
        else:
            if action == "start_encryption":
                success = True
            elif action == "stop_encryption":
                success = True
            else:
                err_msg = f"Invalid encryption action {action}"

    except botocore.exceptions.ClientError as e:
        err_msg = to_native(e)

    return success, err_msg


def retention_action(client, stream_name, retention_period=24, action="increase", check_mode=False):
    """Increase or Decrease the retention of messages in the Kinesis stream.
    Args:
        client (botocore.client.EC2): Boto3 client.
        stream_name (str): The name of the kinesis stream.

    Kwargs:
        retention_period (int): This is how long messages will be kept before
            they are discarded. This can not be less than 24 hours.
        action (str): The action to perform.
            valid actions == create and delete
            default=create
        check_mode (bool): This will pass DryRun as one of the parameters to the aws api.
            default=False

    Basic Usage:
        >>> client = boto3.client('kinesis')
        >>> stream_name = 'test-stream'
        >>> retention_period = 48
        >>> retention_action(client, stream_name, retention_period, action='increase')

    Returns:
        Tuple (bool, str)
    """
    success = False
    err_msg = ""
    params = {"StreamName": stream_name}
    try:
        if not check_mode:
            if action == "increase":
                params["RetentionPeriodHours"] = retention_period
                client.increase_stream_retention_period(**params)
                success = True
                err_msg = f"Retention Period increased successfully to {retention_period}"
            elif action == "decrease":
                params["RetentionPeriodHours"] = retention_period
                client.decrease_stream_retention_period(**params)
                success = True
                err_msg = f"Retention Period decreased successfully to {retention_period}"
            else:
                err_msg = f"Invalid action {action}"
        else:
            if action == "increase":
                success = True
            elif action == "decrease":
                success = True
            else:
                err_msg = f"Invalid action {action}"

    except botocore.exceptions.ClientError as e:
        err_msg = to_native(e)

    return success, err_msg


def update_shard_count(client, stream_name, number_of_shards=1, check_mode=False):
    """Increase or Decrease the number of shards in the Kinesis stream.
    Args:
        client (botocore.client.EC2): Boto3 client.
        stream_name (str): The name of the kinesis stream.

    Kwargs:
        number_of_shards (int): Number of shards this stream will use.
            default=1
        check_mode (bool): This will pass DryRun as one of the parameters to the aws api.
            default=False

    Basic Usage:
        >>> client = boto3.client('kinesis')
        >>> stream_name = 'test-stream'
        >>> number_of_shards = 3
        >>> update_shard_count(client, stream_name, number_of_shards)

    Returns:
        Tuple (bool, str)
    """
    success = True
    err_msg = ""
    params = {"StreamName": stream_name, "ScalingType": "UNIFORM_SCALING"}
    if not check_mode:
        params["TargetShardCount"] = number_of_shards
        try:
            client.update_shard_count(**params)
        except botocore.exceptions.ClientError as e:
            return False, str(e)

    return success, err_msg


def update(
    client,
    current_stream,
    stream_name,
    number_of_shards=1,
    retention_period=None,
    tags=None,
    wait=False,
    wait_timeout=300,
    check_mode=False,
):
    """Update an Amazon Kinesis Stream.
    Args:
        client (botocore.client.EC2): Boto3 client.
        stream_name (str): The name of the kinesis stream.

    Kwargs:
        number_of_shards (int): Number of shards this stream will use.
            default=1
        retention_period (int): This is how long messages will be kept before
            they are discarded. This can not be less than 24 hours.
        tags (dict): The tags you want applied.
        wait (bool): Wait until Stream is ACTIVE.
            default=False
        wait_timeout (int): How long to wait until this operation is considered failed.
            default=300
        check_mode (bool): This will pass DryRun as one of the parameters to the aws api.
            default=False

    Basic Usage:
        >>> client = boto3.client('kinesis')
        >>> current_stream = {
            'ShardCount': 3,
            'HasMoreShards': True,
            'RetentionPeriodHours': 24,
            'StreamName': 'test-stream',
            'StreamARN': 'arn:aws:kinesis:us-west-2:123456789:stream/test-stream',
            'StreamStatus': "ACTIVE'
        }
        >>> stream_name = 'test-stream'
        >>> retention_period = 48
        >>> number_of_shards = 10
        >>> update(client, current_stream, stream_name,
                   number_of_shards, retention_period )

    Returns:
        Tuple (bool, bool, str)
    """
    success = True
    changed = False
    err_msg = ""
    if retention_period:
        if wait:
            wait_success, wait_msg, current_stream = wait_for_status(
                client, stream_name, "ACTIVE", wait_timeout, check_mode=check_mode
            )
            if not wait_success:
                return wait_success, False, wait_msg

        if current_stream.get("StreamStatus") == "ACTIVE":
            retention_changed = False
            if retention_period > current_stream["RetentionPeriodHours"]:
                retention_changed, retention_msg = retention_action(
                    client, stream_name, retention_period, action="increase", check_mode=check_mode
                )

            elif retention_period < current_stream["RetentionPeriodHours"]:
                retention_changed, retention_msg = retention_action(
                    client, stream_name, retention_period, action="decrease", check_mode=check_mode
                )

            elif retention_period == current_stream["RetentionPeriodHours"]:
                retention_msg = f"Retention {retention_period} is the same as {current_stream['RetentionPeriodHours']}"
                success = True

            if retention_changed:
                success = True
                changed = True

            err_msg = retention_msg
            if changed and wait:
                wait_success, wait_msg, current_stream = wait_for_status(
                    client, stream_name, "ACTIVE", wait_timeout, check_mode=check_mode
                )
                if not wait_success:
                    return wait_success, False, wait_msg
            elif changed and not wait:
                stream_found, stream_msg, current_stream = find_stream(client, stream_name)
                if stream_found:
                    if current_stream["StreamStatus"] != "ACTIVE":
                        err_msg = f"Retention Period for {stream_name} is in the process of updating"
                        return success, changed, err_msg
        else:
            err_msg = (
                "StreamStatus has to be ACTIVE in order to modify the retention period."
                f" Current status is {current_stream.get('StreamStatus', 'UNKNOWN')}"
            )
            return success, changed, err_msg

    if current_stream["OpenShardsCount"] != number_of_shards:
        success, err_msg = update_shard_count(client, stream_name, number_of_shards, check_mode=check_mode)

        if not success:
            return success, changed, err_msg

        changed = True

        if wait:
            wait_success, wait_msg, current_stream = wait_for_status(
                client, stream_name, "ACTIVE", wait_timeout, check_mode=check_mode
            )
            if not wait_success:
                return wait_success, changed, wait_msg
        else:
            stream_found, stream_msg, current_stream = find_stream(client, stream_name)
            if stream_found and current_stream["StreamStatus"] != "ACTIVE":
                err_msg = f"Number of shards for {stream_name} is in the process of updating"
                return success, changed, err_msg

    if tags:
        tag_success, tag_changed, err_msg = update_tags(client, stream_name, tags, check_mode=check_mode)
        changed |= tag_changed
    if wait:
        success, err_msg, status_stream = wait_for_status(
            client, stream_name, "ACTIVE", wait_timeout, check_mode=check_mode
        )
    if success and changed:
        err_msg = f"Kinesis Stream {stream_name} updated successfully."
    elif success and not changed:
        err_msg = f"Kinesis Stream {stream_name} did not change."

    return success, changed, err_msg


def create_stream(
    client,
    stream_name,
    number_of_shards=1,
    retention_period=None,
    tags=None,
    wait=False,
    wait_timeout=300,
    check_mode=False,
):
    """Create an Amazon Kinesis Stream.
    Args:
        client (botocore.client.EC2): Boto3 client.
        stream_name (str): The name of the kinesis stream.

    Kwargs:
        number_of_shards (int): Number of shards this stream will use.
            default=1
        retention_period (int): Can not be less than 24 hours
            default=None
        tags (dict): The tags you want applied.
            default=None
        wait (bool): Wait until Stream is ACTIVE.
            default=False
        wait_timeout (int): How long to wait until this operation is considered failed.
            default=300
        check_mode (bool): This will pass DryRun as one of the parameters to the aws api.
            default=False

    Basic Usage:
        >>> client = boto3.client('kinesis')
        >>> stream_name = 'test-stream'
        >>> number_of_shards = 10
        >>> tags = {'env': 'test'}
        >>> create_stream(client, stream_name, number_of_shards, tags=tags)

    Returns:
        Tuple (bool, bool, str, dict)
    """
    success = False
    changed = False
    err_msg = ""
    results = dict()

    stream_found, stream_msg, current_stream = find_stream(client, stream_name)

    if stream_found and current_stream.get("StreamStatus") == "DELETING" and wait:
        wait_success, wait_msg, current_stream = wait_for_status(
            client, stream_name, "ACTIVE", wait_timeout, check_mode=check_mode
        )

    if stream_found and current_stream.get("StreamStatus") != "DELETING":
        success, changed, err_msg = update(
            client,
            current_stream,
            stream_name,
            number_of_shards,
            retention_period,
            tags,
            wait,
            wait_timeout,
            check_mode=check_mode,
        )
    else:
        create_success, create_msg = stream_action(
            client, stream_name, number_of_shards, action="create", check_mode=check_mode
        )
        if not create_success:
            changed = True
            err_msg = f"Failed to create Kinesis stream: {create_msg}"
            return False, True, err_msg, {}
        else:
            changed = True
            if wait:
                wait_success, wait_msg, results = wait_for_status(
                    client, stream_name, "ACTIVE", wait_timeout, check_mode=check_mode
                )
                err_msg = f"Kinesis Stream {stream_name} is in the process of being created"
                if not wait_success:
                    return wait_success, True, wait_msg, results
            else:
                err_msg = f"Kinesis Stream {stream_name} created successfully"

            if tags:
                changed, err_msg = tags_action(client, stream_name, tags, action="create", check_mode=check_mode)
                if changed:
                    success = True
                if not success:
                    return success, changed, err_msg, results

            stream_found, stream_msg, current_stream = find_stream(client, stream_name)
            if retention_period and current_stream.get("StreamStatus") == "ACTIVE":
                changed, err_msg = retention_action(
                    client, stream_name, retention_period, action="increase", check_mode=check_mode
                )
                if changed:
                    success = True
                if not success:
                    return success, changed, err_msg, results
            else:
                err_msg = (
                    "StreamStatus has to be ACTIVE in order to modify the retention period."
                    f" Current status is {current_stream.get('StreamStatus', 'UNKNOWN')}"
                )
                success = create_success
                changed = True

    if success:
        stream_found, stream_msg, results = find_stream(client, stream_name)
        tag_success, tag_msg, current_tags = get_tags(client, stream_name)
        if check_mode:
            current_tags = tags

        if not current_tags:
            current_tags = dict()

        results = camel_dict_to_snake_dict(results)
        results["tags"] = current_tags

    return success, changed, err_msg, results


def delete_stream(client, stream_name, wait=False, wait_timeout=300, check_mode=False):
    """Delete an Amazon Kinesis Stream.
    Args:
        client (botocore.client.EC2): Boto3 client.
        stream_name (str): The name of the kinesis stream.

    Kwargs:
        wait (bool): Wait until Stream is ACTIVE.
            default=False
        wait_timeout (int): How long to wait until this operation is considered failed.
            default=300
        check_mode (bool): This will pass DryRun as one of the parameters to the aws api.
            default=False

    Basic Usage:
        >>> client = boto3.client('kinesis')
        >>> stream_name = 'test-stream'
        >>> delete_stream(client, stream_name)

    Returns:
        Tuple (bool, bool, str, dict)
    """
    success = False
    changed = False
    err_msg = ""
    results = dict()
    stream_found, stream_msg, current_stream = find_stream(client, stream_name)
    if stream_found:
        success, err_msg = stream_action(client, stream_name, action="delete", check_mode=check_mode)
        if success:
            changed = True
            if wait:
                success, err_msg, results = wait_for_status(
                    client, stream_name, "DELETING", wait_timeout, check_mode=check_mode
                )
                err_msg = f"Stream {stream_name} deleted successfully"
                if not success:
                    return success, True, err_msg, results
            else:
                err_msg = f"Stream {stream_name} is in the process of being deleted"
    else:
        success = True
        changed = False
        err_msg = f"Stream {stream_name} does not exist"

    return success, changed, err_msg, results


def start_stream_encryption(
    client, stream_name, encryption_type="", key_id="", wait=False, wait_timeout=300, check_mode=False
):
    """Start encryption on an Amazon Kinesis Stream.
    Args:
        client (botocore.client.EC2): Boto3 client.
        stream_name (str): The name of the kinesis stream.

    Kwargs:
        encryption_type (str): KMS or NONE
        key_id (str): KMS key GUID or alias
        wait (bool): Wait until Stream is ACTIVE.
            default=False
        wait_timeout (int): How long to wait until this operation is considered failed.
            default=300
        check_mode (bool): This will pass DryRun as one of the parameters to the aws api.
            default=False

    Basic Usage:
        >>> client = boto3.client('kinesis')
        >>> stream_name = 'test-stream'
        >>> key_id = 'alias/aws'
        >>> encryption_type = 'KMS'
        >>> start_stream_encryption(client, stream_name,encryption_type,key_id)

    Returns:
        Tuple (bool, bool, str, dict)
    """
    success = False
    changed = False
    err_msg = ""

    results = dict()
    stream_found, stream_msg, current_stream = find_stream(client, stream_name)
    if stream_found:
        if current_stream.get("EncryptionType") == encryption_type and current_stream.get("KeyId") == key_id:
            changed = False
            success = True
            err_msg = f"Kinesis Stream {stream_name} encryption already configured."
        else:
            success, err_msg = stream_encryption_action(
                client,
                stream_name,
                action="start_encryption",
                encryption_type=encryption_type,
                key_id=key_id,
                check_mode=check_mode,
            )
            if success:
                changed = True
                if wait:
                    success, err_msg, results = wait_for_status(
                        client, stream_name, "ACTIVE", wait_timeout, check_mode=check_mode
                    )
                    err_msg = f"Kinesis Stream {stream_name} encryption started successfully."
                    if not success:
                        return success, True, err_msg, results
                else:
                    err_msg = f"Kinesis Stream {stream_name} is in the process of starting encryption."
    else:
        success = True
        changed = False
        err_msg = f"Kinesis Stream {stream_name} does not exist"

    if success:
        stream_found, stream_msg, results = find_stream(client, stream_name)
        tag_success, tag_msg, current_tags = get_tags(client, stream_name)
        if not current_tags:
            current_tags = dict()

        results = camel_dict_to_snake_dict(results)
        results["tags"] = current_tags

    return success, changed, err_msg, results


def stop_stream_encryption(
    client, stream_name, encryption_type="", key_id="", wait=True, wait_timeout=300, check_mode=False
):
    """Stop encryption on an Amazon Kinesis Stream.
    Args:
        client (botocore.client.EC2): Boto3 client.
        stream_name (str): The name of the kinesis stream.

    Kwargs:
        encryption_type (str): KMS or NONE
        key_id (str): KMS key GUID or alias
        wait (bool): Wait until Stream is ACTIVE.
            default=False
        wait_timeout (int): How long to wait until this operation is considered failed.
            default=300
        check_mode (bool): This will pass DryRun as one of the parameters to the aws api.
            default=False

    Basic Usage:
        >>> client = boto3.client('kinesis')
        >>> stream_name = 'test-stream'
        >>> stop_stream_encryption(client, stream_name,encryption_type, key_id)

    Returns:
        Tuple (bool, bool, str, dict)
    """
    success = False
    changed = False
    err_msg = ""

    results = dict()
    stream_found, stream_msg, current_stream = find_stream(client, stream_name)
    if stream_found:
        if current_stream.get("EncryptionType") == "KMS":
            success, err_msg = stream_encryption_action(
                client,
                stream_name,
                action="stop_encryption",
                key_id=key_id,
                encryption_type=encryption_type,
                check_mode=check_mode,
            )
            changed = success
            if wait:
                success, err_msg, results = wait_for_status(
                    client, stream_name, "ACTIVE", wait_timeout, check_mode=check_mode
                )
                if not success:
                    return success, True, err_msg, results
                err_msg = f"Kinesis Stream {stream_name} encryption stopped successfully."
            else:
                err_msg = f"Stream {stream_name} is in the process of stopping encryption."
        elif current_stream.get("EncryptionType") == "NONE":
            success = True
            err_msg = f"Kinesis Stream {stream_name} encryption already stopped."
    else:
        success = True
        changed = False
        err_msg = f"Stream {stream_name} does not exist."

    if success:
        stream_found, stream_msg, results = find_stream(client, stream_name)
        tag_success, tag_msg, current_tags = get_tags(client, stream_name)
        if not current_tags:
            current_tags = dict()

        results = camel_dict_to_snake_dict(results)
        results["tags"] = current_tags

    return success, changed, err_msg, results


def main():
    argument_spec = dict(
        name=dict(required=True),
        shards=dict(default=None, required=False, type="int"),
        retention_period=dict(default=None, required=False, type="int"),
        tags=dict(default=None, required=False, type="dict", aliases=["resource_tags"]),
        wait=dict(default=True, required=False, type="bool"),
        wait_timeout=dict(default=300, required=False, type="int"),
        state=dict(default="present", choices=["present", "absent"]),
        encryption_type=dict(required=False, choices=["NONE", "KMS"]),
        key_id=dict(required=False, type="str"),
        encryption_state=dict(required=False, choices=["enabled", "disabled"]),
    )
    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    retention_period = module.params.get("retention_period")
    stream_name = module.params.get("name")
    shards = module.params.get("shards")
    state = module.params.get("state")
    tags = module.params.get("tags")
    wait = module.params.get("wait")
    wait_timeout = module.params.get("wait_timeout")
    encryption_type = module.params.get("encryption_type")
    key_id = module.params.get("key_id")
    encryption_state = module.params.get("encryption_state")

    if state == "present" and not shards:
        module.fail_json(msg="Shards is required when state == present.")

    if retention_period:
        if retention_period < 24:
            module.fail_json(msg="Retention period can not be less than 24 hours.")

    check_mode = module.check_mode
    try:
        client = module.client("kinesis")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    if state == "present":
        success, changed, err_msg, results = create_stream(
            client, stream_name, shards, retention_period, tags, wait, wait_timeout, check_mode
        )
        if encryption_state == "enabled":
            success, changed, err_msg, results = start_stream_encryption(
                client, stream_name, encryption_type, key_id, wait, wait_timeout, check_mode
            )
        elif encryption_state == "disabled":
            success, changed, err_msg, results = stop_stream_encryption(
                client, stream_name, encryption_type, key_id, wait, wait_timeout, check_mode
            )
    elif state == "absent":
        success, changed, err_msg, results = delete_stream(client, stream_name, wait, wait_timeout, check_mode)

    if success:
        module.exit_json(success=success, changed=changed, msg=err_msg, **results)
    else:
        module.fail_json(success=success, changed=changed, msg=err_msg, result=results)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2025, IP2Location <support@ip2location.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ip2location_info
short_description: Retrieve IP geolocation information of a host's IP address
version_added: 12.2.0
description:
  - Gather IP geolocation information of a host's IP address using the keyless U(api.ip2location.io) API.
author: "IP2Location (@ip2location)"
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.info_module
options:
  ip:
    description:
      - IP address to retrieve geolocation information.
    type: str
  timeout:
    description:
      - HTTP connection timeout in seconds.
    default: 10
    type: int
  http_agent:
    description:
      - Set HTTP user agent.
    default: "ansible-ip2location-info-module/0.0.1"
    type: str
notes:
  - This module uses the keyless endpoint which has usage limits.
    Check U(https://www.ip2location.io/ip2location-documentation) for more information.
"""

EXAMPLES = r"""
# Retrieve geolocation data of a host's IP address
- name: Get IP geolocation data
  community.general.ip2location_info:
  register: result

- name: Show some information
  ansible.builtin.debug:
    msg: "{{ result.ip }} is located in {{ result.country_code }} ({{ result.country_name }})"
"""

RETURN = r"""
ip:
  description: "Public IP address of a host."
  type: str
  returned: success
  sample: "8.8.8.8"
country_code:
  description: ISO 3166-1 alpha-2 country code.
  type: str
  returned: success
  sample: "US"
country_name:
  description: Country name based on ISO 3166.
  type: str
  returned: success
  sample: "United States of America"
region_name:
  description: State or province name.
  type: str
  returned: success
  sample: "California"
city_name:
  description: City name.
  type: str
  returned: success
  sample: "Mountain View"
latitude:
  description: Latitude of the city.
  type: float
  returned: success
  sample: 37.3860
longitude:
  description: Longitude of the city.
  type: float
  returned: success
  sample: -122.0838
zip_code:
  description: ZIP/Postal code.
  type: str
  returned: success
  sample: "94035"
time_zone:
  description: UTC time zone (with DST supported).
  type: str
  returned: success
  sample: "-08:00"
asn:
  description: Autonomous system number (ASN).
  type: str
  returned: success
  sample: "15169"
as:
  description: Autonomous system (AS) name.
  type: str
  returned: success
  sample: "Google LLC"
is_proxy:
  description: Whether is a proxy or not.
  type: bool
  returned: success
  sample: false
"""

import typing as t

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url

USER_AGENT = "ansible-ip2location-info-module/0.0.1"


class Ip2LocationInfo:
    def __init__(self, module: AnsibleModule) -> None:
        self.url = "https://api.ip2location.io/"
        self.timeout = module.params.get("timeout")
        self.ip = module.params.get("ip")
        self.module = module

    def get_geo_data(self) -> dict[str, t.Any]:
        url = self.url
        if self.ip:
            url += f"?ip={self.ip}"

        response, info = fetch_url(
            self.module,
            url,
            timeout=self.timeout,
        )
        if info["status"] != 200:
            self.module.fail_json(
                msg=f"Could not get {url} page, check for connectivity! HTTP Status: {info['status']}"
            )

        try:
            content = response.read()
            result = self.module.from_json(content)
        except Exception as exc:
            self.module.fail_json(
                msg=f"Failed to parse the ip2location.io response from {url}: {exc}", read_content=content
            )
        else:
            return result


def main():
    module = AnsibleModule(
        argument_spec=dict(
            http_agent=dict(default=USER_AGENT),
            timeout=dict(type="int", default=10),
            ip=dict(type="str"),
        ),
        supports_check_mode=True,
    )

    ip2location_info = Ip2LocationInfo(module)
    ip2location_info_result = {"changed": False}
    ip2location_info_result.update(ip2location_info.get_geo_data())
    module.exit_json(**ip2location_info_result)


if __name__ == "__main__":
    main()



#
# Copyright (c) 2017, Daniel Korn <korndaniel1@gmail.com>
#
# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import os
import traceback
import typing as t

from ansible.module_utils.basic import missing_required_lib

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule

CLIENT_IMP_ERR = None
try:
    from manageiq_client.api import ManageIQClient

    HAS_CLIENT = True
except ImportError:
    CLIENT_IMP_ERR = traceback.format_exc()
    HAS_CLIENT = False


def manageiq_argument_spec() -> dict[str, t.Any]:
    options = dict(
        url=dict(default=os.environ.get("MIQ_URL", None)),
        username=dict(default=os.environ.get("MIQ_USERNAME", None)),
        password=dict(default=os.environ.get("MIQ_PASSWORD", None), no_log=True),
        token=dict(default=os.environ.get("MIQ_TOKEN", None), no_log=True),
        validate_certs=dict(default=True, type="bool", aliases=["verify_ssl"]),
        ca_cert=dict(required=False, default=None, aliases=["ca_bundle_path"]),
    )

    return dict(
        manageiq_connection=dict(type="dict", apply_defaults=True, options=options),
    )


def check_client(module: AnsibleModule) -> None:
    if not HAS_CLIENT:
        module.fail_json(msg=missing_required_lib("manageiq-client"), exception=CLIENT_IMP_ERR)


def validate_connection_params(module: AnsibleModule) -> dict[str, t.Any]:
    params: dict[str, t.Any] = module.params["manageiq_connection"]
    url: str | None = params["url"]
    token: str | None = params["token"]
    username: str | None = params["username"]
    password: str | None = params["password"]

    if (url and username and password) or (url and token):
        return params
    for arg in ["url", "username", "password"]:
        if params[arg] in (None, ""):
            module.fail_json(msg=f"missing required argument: manageiq_connection[{arg}]")
    raise AssertionError("should be unreachable")


def manageiq_entities() -> dict[str, str]:
    return {
        "provider": "providers",
        "host": "hosts",
        "vm": "vms",
        "category": "categories",
        "cluster": "clusters",
        "data store": "data_stores",
        "group": "groups",
        "resource pool": "resource_pools",
        "service": "services",
        "service template": "service_templates",
        "template": "templates",
        "tenant": "tenants",
        "user": "users",
        "blueprint": "blueprints",
    }


class ManageIQ:
    """
    class encapsulating ManageIQ API client.
    """

    def __init__(self, module: AnsibleModule) -> None:
        # handle import errors
        check_client(module)

        params = validate_connection_params(module)

        url = params["url"]
        username = params["username"]
        password = params["password"]
        token = params["token"]
        verify_ssl = params["validate_certs"]
        ca_bundle_path = params["ca_cert"]

        self._module = module
        self._api_url = f"{url}/api"
        self._auth = dict(user=username, password=password, token=token)
        try:
            self._client = ManageIQClient(
                self._api_url, self._auth, verify_ssl=verify_ssl, ca_bundle_path=ca_bundle_path
            )
        except Exception as e:
            self.module.fail_json(msg=f"failed to open connection ({url}): {e}")

    @property
    def module(self) -> AnsibleModule:
        """Ansible module module

        Returns:
            the ansible module
        """
        return self._module

    @property
    def api_url(self) -> str:
        """Base ManageIQ API

        Returns:
            the base ManageIQ API
        """
        return self._api_url

    @property
    def client(self):
        """ManageIQ client

        Returns:
            the ManageIQ client
        """
        return self._client

    def find_collection_resource_by(self, collection_name, **params):
        """Searches the collection resource by the collection name and the param passed.

        Returns:
            the resource as an object if it exists in manageiq, None otherwise.
        """
        try:
            entity = self.client.collections.__getattribute__(collection_name).get(**params)
        except ValueError:
            return None
        except Exception as e:
            self.module.fail_json(msg=f"failed to find resource {e}")
        return vars(entity)

    def find_collection_resource_or_fail(self, collection_name, **params):
        """Searches the collection resource by the collection name and the param passed.

        Returns:
            the resource as an object if it exists in manageiq, Fail otherwise.
        """
        resource = self.find_collection_resource_by(collection_name, **params)
        if resource:
            return resource
        else:
            msg = f"{collection_name} where {params} does not exist in manageiq"
            self.module.fail_json(msg=msg)

    def policies(self, resource_id, resource_type, resource_name):
        manageiq = ManageIQ(self.module)

        # query resource id, fail if resource does not exist
        if resource_id is None:
            resource_id = manageiq.find_collection_resource_or_fail(resource_type, name=resource_name)["id"]

        return ManageIQPolicies(manageiq, resource_type, resource_id)

    def query_resource_id(self, resource_type, resource_name):
        """Query the resource name in ManageIQ.

        Returns:
            the resource ID if it exists in ManageIQ, Fail otherwise.
        """
        resource = self.find_collection_resource_by(resource_type, name=resource_name)
        if resource:
            return resource["id"]
        else:
            msg = f"{resource_name} {resource_type} does not exist in manageiq"
            self.module.fail_json(msg=msg)


class ManageIQPolicies:
    """
    Object to execute policies management operations of manageiq resources.
    """

    def __init__(self, manageiq: ManageIQ, resource_type, resource_id):
        self.manageiq = manageiq

        self.module = self.manageiq.module
        self.api_url = self.manageiq.api_url
        self.client = self.manageiq.client

        self.resource_type = resource_type
        self.resource_id = resource_id
        self.resource_url = f"{self.api_url}/{resource_type}/{resource_id}"

    def query_profile_href(self, profile):
        """Add or Update the policy_profile href field

        Example:
            {name: STR, ...} => {name: STR, href: STR}
        """
        resource = self.manageiq.find_collection_resource_or_fail("policy_profiles", **profile)
        return dict(name=profile["name"], href=resource["href"])

    def query_resource_profiles(self):
        """Returns a set of the profile objects objects assigned to the resource"""
        url = f"{self.resource_url}/policy_profiles?expand=resources"
        try:
            response = self.client.get(url)
        except Exception as e:
            msg = f"Failed to query {self.resource_type} policies: {e}"
            self.module.fail_json(msg=msg)

        resources = response.get("resources", [])

        # clean the returned rest api profile object to look like:
        # {profile_name: STR, profile_description: STR, policies: ARR<POLICIES>}
        profiles = [self.clean_profile_object(profile) for profile in resources]

        return profiles

    def query_profile_policies(self, profile_id):
        """Returns a set of the policy objects assigned to the resource"""
        url = f"{self.api_url}/policy_profiles/{profile_id}?expand=policies"
        try:
            response = self.client.get(url)
        except Exception as e:
            msg = f"Failed to query {self.resource_type} policies: {e}"
            self.module.fail_json(msg=msg)

        resources = response.get("policies", [])

        # clean the returned rest api policy object to look like:
        # {name: STR, description: STR, active: BOOL}
        policies = [self.clean_policy_object(policy) for policy in resources]

        return policies

    def clean_policy_object(self, policy):
        """Clean a policy object to have human readable form of:
        {
            name: STR,
            description: STR,
            active: BOOL
        }
        """
        name = policy.get("name")
        description = policy.get("description")
        active = policy.get("active")

        return dict(name=name, description=description, active=active)

    def clean_profile_object(self, profile):
        """Clean a profile object to have human readable form of:
        {
            profile_name: STR,
            profile_description: STR,
            policies: ARR<POLICIES>
        }
        """
        profile_id = profile["id"]
        name = profile.get("name")
        description = profile.get("description")
        policies = self.query_profile_policies(profile_id)

        return dict(profile_name=name, profile_description=description, policies=policies)

    def profiles_to_update(self, profiles, action):
        """Create a list of policies we need to update in ManageIQ.

        Returns:
            Whether or not a change took place and a message describing the
            operation executed.
        """
        profiles_to_post = []
        assigned_profiles = self.query_resource_profiles()

        # make a list of assigned full profile names strings
        # e.g. ['openscap profile', ...]
        assigned_profiles_set = {profile["profile_name"] for profile in assigned_profiles}

        for profile in profiles:
            assigned = profile.get("name") in assigned_profiles_set

            if (action == "unassign" and assigned) or (action == "assign" and not assigned):
                # add/update the policy profile href field
                # {name: STR, ...} => {name: STR, href: STR}
                profile = self.query_profile_href(profile)
                profiles_to_post.append(profile)

        return profiles_to_post

    def assign_or_unassign_profiles(self, profiles, action):
        """Perform assign/unassign action"""
        # get a list of profiles needed to be changed
        profiles_to_post = self.profiles_to_update(profiles, action)
        if not profiles_to_post:
            return dict(changed=False, msg=f"Profiles {profiles} already {action}ed, nothing to do")

        # try to assign or unassign profiles to resource
        url = f"{self.resource_url}/policy_profiles"
        try:
            response = self.client.post(url, action=action, resources=profiles_to_post)
        except Exception as e:
            msg = f"Failed to {action} profile: {e}"
            self.module.fail_json(msg=msg)

        # check all entities in result to be successful
        for result in response["results"]:
            if not result["success"]:
                msg = f"Failed to {action}: {result['message']}"
                self.module.fail_json(msg=msg)

        # successfully changed all needed profiles
        return dict(changed=True, msg=f"Successfully {action}ed profiles: {profiles}")


class ManageIQTags:
    """
    Object to execute tags management operations of manageiq resources.
    """

    def __init__(self, manageiq: ManageIQ, resource_type, resource_id):
        self.manageiq = manageiq

        self.module = self.manageiq.module
        self.api_url = self.manageiq.api_url
        self.client = self.manageiq.client

        self.resource_type = resource_type
        self.resource_id = resource_id
        self.resource_url = f"{self.api_url}/{resource_type}/{resource_id}"

    def full_tag_name(self, tag):
        """Returns the full tag name in manageiq"""
        return f"/managed/{tag['category']}/{tag['name']}"

    def clean_tag_object(self, tag):
        """Clean a tag object to have human readable form of:
        {
            full_name: STR,
            name: STR,
            display_name: STR,
            category: STR
        }
        """
        full_name = tag.get("name")
        categorization = tag.get("categorization", {})

        return dict(
            full_name=full_name,
            name=categorization.get("name"),
            display_name=categorization.get("display_name"),
            category=categorization.get("category", {}).get("name"),
        )

    def query_resource_tags(self):
        """Returns a set of the tag objects assigned to the resource"""
        url = f"{self.resource_url}/tags?expand=resources&attributes=categorization"
        try:
            response = self.client.get(url)
        except Exception as e:
            msg = f"Failed to query {self.resource_type} tags: {e}"
            self.module.fail_json(msg=msg)

        resources = response.get("resources", [])

        # clean the returned rest api tag object to look like:
        # {full_name: STR, name: STR, display_name: STR, category: STR}
        tags = [self.clean_tag_object(tag) for tag in resources]

        return tags

    def tags_to_update(self, tags, action):
        """Create a list of tags we need to update in ManageIQ.

        Returns:
            Whether or not a change took place and a message describing the
            operation executed.
        """
        tags_to_post = []
        assigned_tags = self.query_resource_tags()

        # make a list of assigned full tag names strings
        # e.g. ['/managed/environment/prod', ...]
        assigned_tags_set = {tag["full_name"] for tag in assigned_tags}

        for tag in tags:
            assigned = self.full_tag_name(tag) in assigned_tags_set

            if assigned and action == "unassign":
                tags_to_post.append(tag)
            elif (not assigned) and action == "assign":
                tags_to_post.append(tag)

        return tags_to_post

    def assign_or_unassign_tags(self, tags, action):
        """Perform assign/unassign action"""
        # get a list of tags needed to be changed
        tags_to_post = self.tags_to_update(tags, action)
        if not tags_to_post:
            return dict(changed=False, msg=f"Tags already {action}ed, nothing to do")

        # try to assign or unassign tags to resource
        url = f"{self.resource_url}/tags"
        try:
            response = self.client.post(url, action=action, resources=tags)
        except Exception as e:
            msg = f"Failed to {action} tag: {e}"
            self.module.fail_json(msg=msg)

        # check all entities in result to be successful
        for result in response["results"]:
            if not result["success"]:
                msg = f"Failed to {action}: {result['message']}"
                self.module.fail_json(msg=msg)

        # successfully changed all needed tags
        return dict(changed=True, msg=f"Successfully {action}ed tags")



#!/usr/bin/python

# Copyright (c) 2021, Alexei Znamensky (@russoz) <russoz@gmail.com>
# Copyright (c) 2013, Matthias Vogelgesang <matthias.vogelgesang@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: kernel_blacklist
author:
  - Matthias Vogelgesang (@matze)
short_description: Blacklist kernel modules
description:
  - Add or remove kernel modules from blacklist.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
options:
  name:
    type: str
    description:
      - Name of kernel module to black- or whitelist.
    required: true
  state:
    type: str
    description:
      - Whether the module should be present in the blacklist or absent.
    choices: [absent, present]
    default: present
  blacklist_file:
    type: str
    description:
      - If specified, use this blacklist file instead of C(/etc/modprobe.d/blacklist-ansible.conf).
    default: /etc/modprobe.d/blacklist-ansible.conf
"""

EXAMPLES = r"""
- name: Blacklist the nouveau driver module
  community.general.kernel_blacklist:
    name: nouveau
    state: present
"""

import os
import re

from ansible_collections.community.general.plugins.module_utils._module_helper import StateModuleHelper


class Blacklist(StateModuleHelper):
    output_params = ("name", "state")
    module = dict(
        argument_spec=dict(
            name=dict(type="str", required=True),
            state=dict(type="str", default="present", choices=["absent", "present"]),
            blacklist_file=dict(type="str", default="/etc/modprobe.d/blacklist-ansible.conf"),
        ),
        supports_check_mode=True,
    )

    def __init_module__(self):
        self.pattern = re.compile(rf"^blacklist\s+{re.escape(self.vars.name)}$")
        self.vars.filename = self.vars.blacklist_file
        self.vars.set("file_exists", os.path.exists(self.vars.filename), output=False, change=True)
        if not self.vars.file_exists:
            with open(self.vars.filename, "a"):
                pass
            self.vars.file_exists = True
            self.vars.set("lines", [], change=True, diff=True)
        else:
            with open(self.vars.filename) as fd:
                self.vars.set("lines", [x.rstrip() for x in fd.readlines()], change=True, diff=True)
        self.vars.set("is_blacklisted", self._is_module_blocked(), change=True)

    def _is_module_blocked(self):
        for line in self.vars.lines:
            stripped = line.strip()
            if stripped.startswith("#"):
                continue
            if self.pattern.match(stripped):
                return True
        return False

    def state_absent(self):
        if not self.vars.is_blacklisted:
            return
        self.vars.is_blacklisted = False
        self.vars.lines = [line for line in self.vars.lines if not self.pattern.match(line.strip())]

    def state_present(self):
        if self.vars.is_blacklisted:
            return
        self.vars.is_blacklisted = True
        self.vars.lines = self.vars.lines + [f"blacklist {self.vars.name}"]

    def __quit_module__(self):
        if self.has_changed() and not self.module.check_mode:
            bkp = self.module.backup_local(self.vars.filename)
            with open(self.vars.filename, "w") as fd:
                fd.writelines([f"{x}\n" for x in self.vars.lines])
            self.module.add_cleanup_file(bkp)


def main():
    Blacklist.execute()


if __name__ == "__main__":
    main()



# Copyright (c) 2024 Vladimir Botka <vbotka@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: reveal_ansible_type
short_description: Return input type
version_added: "9.2.0"
author: Vladimir Botka (@vbotka)
description: This filter returns input type.
options:
  _input:
    description: Input data.
    type: raw
    required: true
  alias:
    description: Data type aliases.
    default: {}
    type: dictionary
"""

EXAMPLES = r"""
# Substitution converts str to AnsibleUnicode or _AnsibleTaggedStr
# ----------------------------------------------------------------

# String. AnsibleUnicode or _AnsibleTaggedStr.
- data: "abc"
  result: '{{ data | community.general.reveal_ansible_type }}'
# result => AnsibleUnicode (or _AnsibleTaggedStr)

# String. AnsibleUnicode/_AnsibleTaggedStr alias str.
- alias: {"AnsibleUnicode": "str", "_AnsibleTaggedStr": "str"}
  data: "abc"
  result: '{{ data | community.general.reveal_ansible_type(alias) }}'
# result => str

# List. All items are AnsibleUnicode/_AnsibleTaggedStr.
- data: ["a", "b", "c"]
  result: '{{ data | community.general.reveal_ansible_type }}'
# result => list[AnsibleUnicode] or list[_AnsibleTaggedStr]

# Dictionary. All keys and values are AnsibleUnicode/_AnsibleTaggedStr.
- data: {"a": "foo", "b": "bar", "c": "baz"}
  result: '{{ data | community.general.reveal_ansible_type }}'
# result => dict[AnsibleUnicode, AnsibleUnicode] or dict[_AnsibleTaggedStr, _AnsibleTaggedStr]

# No substitution and no alias. Type of strings is str
# ----------------------------------------------------

# String
- result: '{{ "abc" | community.general.reveal_ansible_type }}'
# result => str

# Integer
- result: '{{ 123 | community.general.reveal_ansible_type }}'
# result => int

# Float
- result: '{{ 123.45 | community.general.reveal_ansible_type }}'
# result => float

# Boolean
- result: '{{ true | community.general.reveal_ansible_type }}'
# result => bool

# List. All items are strings.
- result: '{{ ["a", "b", "c"] | community.general.reveal_ansible_type }}'
# result => list[str]

# List of dictionaries.
- result: '{{ [{"a": 1}, {"b": 2}] | community.general.reveal_ansible_type }}'
# result => list[dict]

# Dictionary. All keys are strings. All values are integers.
- result: '{{ {"a": 1} | community.general.reveal_ansible_type }}'
# result => dict[str, int]

# Dictionary. All keys are strings. All values are integers.
- result: '{{ {"a": 1, "b": 2} | community.general.reveal_ansible_type }}'
# result => dict[str, int]

# Type of strings is AnsibleUnicode, _AnsibleTaggedStr, or str
# ------------------------------------------------------------

# Dictionary. The keys are integers or strings. All values are strings.
- alias:
    AnsibleUnicode: str
    _AnsibleTaggedStr: str
    _AnsibleTaggedInt: int
  data: {1: 'a', 'b': 'b'}
  result: '{{ data | community.general.reveal_ansible_type(alias) }}'
# result => dict[int|str, str]

# Dictionary. All keys are integers. All values are keys.
- alias:
    AnsibleUnicode: str
    _AnsibleTaggedStr: str
    _AnsibleTaggedInt: int
  data: {1: 'a', 2: 'b'}
  result: '{{ data | community.general.reveal_ansible_type(alias) }}'
# result => dict[int, str]

# Dictionary. All keys are strings. Multiple types values.
- alias:
    AnsibleUnicode: str
    _AnsibleTaggedStr: str
    _AnsibleTaggedInt: int
    _AnsibleTaggedFloat: float
  data: {'a': 1, 'b': 1.1, 'c': 'abc', 'd': true, 'e': ['x', 'y', 'z'], 'f': {'x': 1, 'y': 2}}
  result: '{{ data | community.general.reveal_ansible_type(alias) }}'
# result => dict[str, bool|dict|float|int|list|str]

# List. Multiple types items.
- alias:
    AnsibleUnicode: str
    _AnsibleTaggedStr: str
    _AnsibleTaggedInt: int
    _AnsibleTaggedFloat: float
  data: [1, 2, 1.1, 'abc', true, ['x', 'y', 'z'], {'x': 1, 'y': 2}]
  result: '{{ data | community.general.reveal_ansible_type(alias) }}'
# result => list[bool|dict|float|int|list|str]
"""

RETURN = r"""
_value:
  description: Type of the data.
  type: str
"""

from ansible_collections.community.general.plugins.plugin_utils._ansible_type import _ansible_type


def reveal_ansible_type(data, alias=None):
    """Returns data type"""

    # TODO: expose use_native_type parameter
    return _ansible_type(data, alias)


class FilterModule:
    def filters(self):
        return {"reveal_ansible_type": reveal_ansible_type}



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_mds
short_description: Manages mds objects on Checkpoint over Web Services API
description:
  - Manages mds objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R80.40 JHF management version.
version_added: "2.1.0"
author: "Or Soffer (@chkp-orso)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  ip_address:
    description:
      - IPv4 or IPv6 address. If both addresses are required use ipv4-address and ipv6-address fields explicitly.
    type: str
  ipv4_address:
    description:
      - IPv4 address.
    type: str
  ipv6_address:
    description:
      - IPv6 address.
    type: str
  hardware:
    description:
      - Hardware name. For example, Open server, Smart-1, Other.
    type: str
  os:
    description:
      - Operating system name. For example, Gaia, Linux, SecurePlatform.
    type: str
  version:
    description:
      - System version.
    type: str
  one_time_password:
    description:
      - Secure internal connection one time password.
    type: str
  server_type:
    description:
      - Type of the management server.
    type: str
    choices: ['multi-domain server', 'multi-domain log server']
  ip_pool_first:
    description:
      - First IP address in the range.
    type: str
  ipv4_pool_first:
    description:
      - First IPv4 address in the range.
    type: str
  ipv6_pool_first:
    description:
      - First IPv6 address in the range.
    type: str
  ip_pool_last:
    description:
      - Last IP address in the range.
    type: str
  ipv4_pool_last:
    description:
      - Last IPv4 address in the range.
    type: str
  ipv6_pool_last:
    description:
      - Last IPv6 address in the range.
    type: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-mds
  cp_mgmt_mds:
    hardware: open server
    ip_address: 1.1.1.1
    ip_pool_first: 2.2.2.2
    ip_pool_last: 3.3.3.3
    name: mymds
    os: gaia
    server_type: multi-domain server
    state: present

- name: set-mds
  cp_mgmt_mds:
    hardware: Smart-1
    ip_address: 1.2.3.4
    name: mymds
    os: linux
    state: present

- name: delete-mds
  cp_mgmt_mds:
    name: mymds
    state: absent
"""

RETURN = """
cp_mgmt_mds:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_objects,
    api_call,
)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        ip_address=dict(type="str"),
        ipv4_address=dict(type="str"),
        ipv6_address=dict(type="str"),
        hardware=dict(type="str"),
        os=dict(type="str"),
        version=dict(type="str"),
        one_time_password=dict(type="str", no_log=True),
        server_type=dict(
            type="str",
            choices=["multi-domain server", "multi-domain log server"],
        ),
        ip_pool_first=dict(type="str"),
        ipv4_pool_first=dict(type="str"),
        ipv6_pool_first=dict(type="str"),
        ip_pool_last=dict(type="str"),
        ipv4_pool_last=dict(type="str"),
        ipv6_pool_last=dict(type="str"),
        tags=dict(type="list", elements="str"),
        color=dict(
            type="str",
            choices=[
                "aquamarine",
                "black",
                "blue",
                "crete blue",
                "burlywood",
                "cyan",
                "dark green",
                "khaki",
                "orchid",
                "dark orange",
                "dark sea green",
                "pink",
                "turquoise",
                "dark blue",
                "firebrick",
                "brown",
                "forest green",
                "gold",
                "dark gold",
                "gray",
                "dark gray",
                "light green",
                "lemon chiffon",
                "coral",
                "sea green",
                "sky blue",
                "magenta",
                "purple",
                "slate blue",
                "violet red",
                "navy blue",
                "olive",
                "orange",
                "red",
                "sienna",
                "yellow",
            ],
        ),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )
    api_call_object = "mds"

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_qos_dscp_class
short_description: Manage QoS DSCP Class (qos:DscpClass)
description:
- Manage QoS Custom Differentiated Services Code Point (DSCP) Class levels for QoS Custom Policies on Cisco ACI fabrics.
- The class level for DSCP to prioritize the map.
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  qos_custom_policy:
    description:
    - The name of the QoS Custom Policy.
    type: str
    aliases: [ qos_custom_policy_name ]
  priority:
    description:
    - The desired QoS class level to be used.
    - The APIC defaults to C(level3) when unset during creation.
    type: str
    choices: [ level1, level2, level3, level4, level5, level6, unspecified ]
    aliases: [ prio ]
  dscp_from:
    description:
    - The DSCP range starting value.
    type: str
    choices: [ AF11, AF12, AF13, AF21, AF22, AF23, AF31, AF32, AF33, AF41, AF42, AF43, CS0, CS1, CS2, CS3, CS4, CS5, CS6, CS7, EF, VA, unspecified ]
  dscp_to:
    description:
    - The DSCP range ending value.
    type: str
    choices: [ AF11, AF12, AF13, AF21, AF22, AF23, AF31, AF32, AF33, AF41, AF42, AF43, CS0, CS1, CS2, CS3, CS4, CS5, CS6, CS7, EF, VA, unspecified ]
  dscp_target:
    description:
    - The DSCP target value.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ AF11, AF12, AF13, AF21, AF22, AF23, AF31, AF32, AF33, AF41, AF42, AF43, CS0, CS1, CS2, CS3, CS4, CS5, CS6, CS7, EF, VA, unspecified ]
    aliases: [ target ]
  target_cos:
    description:
    - The target COS to be driven based on the range of input values of DSCP coming into the fabric.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ background, best_effort, excellent_effort, critical_applications, video, voice, internetwork_control, network_control, unspecified ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The I(tenant) and I(qos_custom_policy) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) and the M(cisco.aci.aci_qos_custom_policy) can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_qos_custom_policy
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(qos:DscpClass).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Gaspard Micol (@gmicol)
"""

EXAMPLES = r"""
- name: Add a new QoS DSCP Class
  cisco.aci.aci_qos_dscp_class:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    qos_custom_policy: my_qos_custom_policy
    priority: level3
    dscp_from: AF11
    dscp_to: AF21
    dscp_target: unspecified
    target_cos: best_effort
    state: present
  delegate_to: localhost

- name: Query a QoS DSCP Class
  cisco.aci.aci_qos_dscp_class:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    qos_custom_policy: my_qos_custom_policy
    dscp_from: AF11
    dscp_to: AF21
    state: query
  delegate_to: localhost

- name: Query all QoS DSCP Classes in my_qos_custom_policy
  cisco.aci.aci_qos_dscp_class:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    qos_custom_policy: my_qos_custom_policy
    state: query
  delegate_to: localhost

- name: Query all QoS DSCP Classes
  cisco.aci.aci_qos_dscp_class:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost

- name: Delete a QoS DSCP Class
  cisco.aci.aci_qos_dscp_class:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    qos_custom_policy: my_qos_custom_policy
    dscp_from: AF11
    dscp_to: AF21
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import (
    ACIModule,
    aci_argument_spec,
    aci_annotation_spec,
    aci_owner_spec,
    aci_contract_dscp_spec,
)
from ansible_collections.cisco.aci.plugins.module_utils.constants import MATCH_TARGET_COS_MAPPING


def main():
    new_dscp_spec = dict((k, aci_contract_dscp_spec()[k]) for k in aci_contract_dscp_spec() if k != "aliases")
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        qos_custom_policy=dict(type="str", aliases=["qos_custom_policy_name"]),
        priority=dict(
            type="str",
            choices=[
                "level1",
                "level2",
                "level3",
                "level4",
                "level5",
                "level6",
                "unspecified",
            ],
            aliases=["prio"],
        ),
        dscp_from=new_dscp_spec,
        dscp_to=new_dscp_spec,
        dscp_target=aci_contract_dscp_spec(),
        target_cos=dict(type="str", choices=list(MATCH_TARGET_COS_MAPPING.keys())),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "qos_custom_policy", "dscp_from", "dscp_to"]],
            ["state", "present", ["tenant", "qos_custom_policy", "dscp_from", "dscp_to"]],
        ],
    )

    tenant = module.params.get("tenant")
    qos_custom_policy = module.params.get("qos_custom_policy")
    priority = module.params.get("priority")
    dscp_from = module.params.get("dscp_from")
    dscp_to = module.params.get("dscp_to")
    dscp_target = module.params.get("dscp_target")
    target_cos = MATCH_TARGET_COS_MAPPING.get(module.params.get("target_cos"))
    state = module.params.get("state")

    aci = ACIModule(module)

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="qosCustomPol",
            aci_rn="qoscustom-{0}".format(qos_custom_policy),
            module_object=qos_custom_policy,
            target_filter={"name": qos_custom_policy},
        ),
        subclass_2=dict(
            aci_class="qosDscpClass",
            aci_rn="dcsp-{0}-{1}".format(dscp_from, dscp_to),
            module_object=qos_custom_policy,
            target_filter={"from": dscp_from, "to": dscp_to},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="qosDscpClass",
            class_config={
                "prio": priority,
                "from": dscp_from,
                "to": dscp_to,
                "target": dscp_target,
                "targetCos": target_cos,
            },
        )

        aci.get_diff(aci_class="qosDscpClass")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_wireless_rf_profiles_assignments_by_device_info
short_description: Information module for organizations _wireless _rf _profiles _assignments
  _by _device
description:
  - Get all organizations _wireless _rf _profiles _assignments _by _device.
  - List the RF profiles of an organization by device.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  networkIds:
    description:
      - NetworkIds query parameter. Optional parameter to filter devices by network.
    elements: str
    type: list
  productTypes:
    description:
      - >
        ProductTypes query parameter. Optional parameter to filter devices by product
        type. Valid types are wireless, appliance, switch, systemsManager, camera,
        cellularGateway, sensor, wirelessController, campusGateway, and secureConnect.
    elements: str
    type: list
  name:
    description:
      - >
        Name query parameter. Optional parameter to filter RF profiles by device name.
        All returned devices will have a name that contains the search term or is
        an exact match.
    type: str
  mac:
    description:
      - >
        Mac query parameter. Optional parameter to filter RF profiles by device MAC
        address. All returned devices will have a MAC address that contains the search
        term or is an exact match.
    type: str
  serial:
    description:
      - >
        Serial query parameter. Optional parameter to filter RF profiles by device
        serial number. All returned devices will have a serial number that contains
        the search term or is an exact match.
    type: str
  model:
    description:
      - >
        Model query parameter. Optional parameter to filter RF profiles by device
        model. All returned devices will have a model that contains the search term
        or is an exact match.
    type: str
  macs:
    description:
      - >
        Macs query parameter. Optional parameter to filter RF profiles by one or more
        device MAC addresses. All returned devices will have a MAC address that is
        an exact match.
    elements: str
    type: list
  serials:
    description:
      - >
        Serials query parameter. Optional parameter to filter RF profiles by one or
        more device serial numbers. All returned devices will have a serial number
        that is an exact match.
    elements: str
    type: list
  models:
    description:
      - >
        Models query parameter. Optional parameter to filter RF profiles by one or
        more device models. All returned devices will have a model that is an exact
        match.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless getOrganizationWirelessRfProfilesAssignmentsByDevice
    description: Complete reference of the getOrganizationWirelessRfProfilesAssignmentsByDevice
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-wireless-rf-profiles-assignments-by-device
notes:
  - SDK Method used are
    wireless.Wireless.get_organization_wireless_rf_profiles_assignments_by_device,
  - Paths used are
    get /organizations/{organizationId}/wireless/rfProfiles/assignments/byDevice,
"""

EXAMPLES = r"""
- name: Get all organizations _wireless _rf _profiles _assignments _by _device
  cisco.meraki.organizations_wireless_rf_profiles_assignments_by_device_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    networkIds: []
    productTypes: []
    name: string
    mac: string
    serial: string
    model: string
    macs: []
    serials: []
    models: []
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "items": [
          {
            "network": {
              "id": "string"
            },
            "name": "string",
            "serial": "string",
            "model": "string",
            "rfProfile": {
              "id": "string",
              "name": "string",
              "isIndoorDefault": true,
              "isOutdoorDefault": true
            }
          }
        ],
        "meta": {
          "counts": {
            "items": {
              "total": 0,
              "remaining": 0
            }
          }
        }
      }
    ]
"""
